
from sysactions import get_model, get_posot
from xml.etree import cElementTree as eTree


def synchronize_ui(
    pos_id,         # type: str
    model=None,     # type: eTree.Element

):
    # type: (...) -> None

    if model is None:
        model = get_model(posid=pos_id)

    pos_ot = get_posot(model=model)

    sale_type = model.find(".//Order").get("saleType")
    pos_ot.updateOrderProperties(
        posid=pos_id,
        saletype=sale_type,
    )



data = (
'sswals',    # 0x00
'sswalt',    # 0x01
'sswalp',    # 0x02
'sswalh',    # 0x03
'sswam',    # 0x04
'sswab',    # 0x05
'sswabs',    # 0x06
'sswas',    # 0x07
'sswass',    # 0x08
'sswang',    # 0x09
'sswaj',    # 0x0a
'sswac',    # 0x0b
'sswak',    # 0x0c
'sswat',    # 0x0d
'sswap',    # 0x0e
'sswah',    # 0x0f
'sswae',    # 0x10
'sswaeg',    # 0x11
'sswaegg',    # 0x12
'sswaegs',    # 0x13
'sswaen',    # 0x14
'sswaenj',    # 0x15
'sswaenh',    # 0x16
'sswaed',    # 0x17
'sswael',    # 0x18
'sswaelg',    # 0x19
'sswaelm',    # 0x1a
'sswaelb',    # 0x1b
'sswaels',    # 0x1c
'sswaelt',    # 0x1d
'sswaelp',    # 0x1e
'sswaelh',    # 0x1f
'sswaem',    # 0x20
'sswaeb',    # 0x21
'sswaebs',    # 0x22
'sswaes',    # 0x23
'sswaess',    # 0x24
'sswaeng',    # 0x25
'sswaej',    # 0x26
'sswaec',    # 0x27
'sswaek',    # 0x28
'sswaet',    # 0x29
'sswaep',    # 0x2a
'sswaeh',    # 0x2b
'ssoe',    # 0x2c
'ssoeg',    # 0x2d
'ssoegg',    # 0x2e
'ssoegs',    # 0x2f
'ssoen',    # 0x30
'ssoenj',    # 0x31
'ssoenh',    # 0x32
'ssoed',    # 0x33
'ssoel',    # 0x34
'ssoelg',    # 0x35
'ssoelm',    # 0x36
'ssoelb',    # 0x37
'ssoels',    # 0x38
'ssoelt',    # 0x39
'ssoelp',    # 0x3a
'ssoelh',    # 0x3b
'ssoem',    # 0x3c
'ssoeb',    # 0x3d
'ssoebs',    # 0x3e
'ssoes',    # 0x3f
'ssoess',    # 0x40
'ssoeng',    # 0x41
'ssoej',    # 0x42
'ssoec',    # 0x43
'ssoek',    # 0x44
'ssoet',    # 0x45
'ssoep',    # 0x46
'ssoeh',    # 0x47
'ssyo',    # 0x48
'ssyog',    # 0x49
'ssyogg',    # 0x4a
'ssyogs',    # 0x4b
'ssyon',    # 0x4c
'ssyonj',    # 0x4d
'ssyonh',    # 0x4e
'ssyod',    # 0x4f
'ssyol',    # 0x50
'ssyolg',    # 0x51
'ssyolm',    # 0x52
'ssyolb',    # 0x53
'ssyols',    # 0x54
'ssyolt',    # 0x55
'ssyolp',    # 0x56
'ssyolh',    # 0x57
'ssyom',    # 0x58
'ssyob',    # 0x59
'ssyobs',    # 0x5a
'ssyos',    # 0x5b
'ssyoss',    # 0x5c
'ssyong',    # 0x5d
'ssyoj',    # 0x5e
'ssyoc',    # 0x5f
'ssyok',    # 0x60
'ssyot',    # 0x61
'ssyop',    # 0x62
'ssyoh',    # 0x63
'ssu',    # 0x64
'ssug',    # 0x65
'ssugg',    # 0x66
'ssugs',    # 0x67
'ssun',    # 0x68
'ssunj',    # 0x69
'ssunh',    # 0x6a
'ssud',    # 0x6b
'ssul',    # 0x6c
'ssulg',    # 0x6d
'ssulm',    # 0x6e
'ssulb',    # 0x6f
'ssuls',    # 0x70
'ssult',    # 0x71
'ssulp',    # 0x72
'ssulh',    # 0x73
'ssum',    # 0x74
'ssub',    # 0x75
'ssubs',    # 0x76
'ssus',    # 0x77
'ssuss',    # 0x78
'ssung',    # 0x79
'ssuj',    # 0x7a
'ssuc',    # 0x7b
'ssuk',    # 0x7c
'ssut',    # 0x7d
'ssup',    # 0x7e
'ssuh',    # 0x7f
'ssweo',    # 0x80
'ssweog',    # 0x81
'ssweogg',    # 0x82
'ssweogs',    # 0x83
'ssweon',    # 0x84
'ssweonj',    # 0x85
'ssweonh',    # 0x86
'ssweod',    # 0x87
'ssweol',    # 0x88
'ssweolg',    # 0x89
'ssweolm',    # 0x8a
'ssweolb',    # 0x8b
'ssweols',    # 0x8c
'ssweolt',    # 0x8d
'ssweolp',    # 0x8e
'ssweolh',    # 0x8f
'ssweom',    # 0x90
'ssweob',    # 0x91
'ssweobs',    # 0x92
'ssweos',    # 0x93
'ssweoss',    # 0x94
'ssweong',    # 0x95
'ssweoj',    # 0x96
'ssweoc',    # 0x97
'ssweok',    # 0x98
'ssweot',    # 0x99
'ssweop',    # 0x9a
'ssweoh',    # 0x9b
'sswe',    # 0x9c
'ssweg',    # 0x9d
'sswegg',    # 0x9e
'sswegs',    # 0x9f
'sswen',    # 0xa0
'sswenj',    # 0xa1
'sswenh',    # 0xa2
'sswed',    # 0xa3
'sswel',    # 0xa4
'sswelg',    # 0xa5
'sswelm',    # 0xa6
'sswelb',    # 0xa7
'sswels',    # 0xa8
'sswelt',    # 0xa9
'sswelp',    # 0xaa
'sswelh',    # 0xab
'sswem',    # 0xac
'ssweb',    # 0xad
'sswebs',    # 0xae
'sswes',    # 0xaf
'sswess',    # 0xb0
'ssweng',    # 0xb1
'sswej',    # 0xb2
'sswec',    # 0xb3
'sswek',    # 0xb4
'sswet',    # 0xb5
'sswep',    # 0xb6
'ssweh',    # 0xb7
'sswi',    # 0xb8
'sswig',    # 0xb9
'sswigg',    # 0xba
'sswigs',    # 0xbb
'sswin',    # 0xbc
'sswinj',    # 0xbd
'sswinh',    # 0xbe
'sswid',    # 0xbf
'sswil',    # 0xc0
'sswilg',    # 0xc1
'sswilm',    # 0xc2
'sswilb',    # 0xc3
'sswils',    # 0xc4
'sswilt',    # 0xc5
'sswilp',    # 0xc6
'sswilh',    # 0xc7
'sswim',    # 0xc8
'sswib',    # 0xc9
'sswibs',    # 0xca
'sswis',    # 0xcb
'sswiss',    # 0xcc
'sswing',    # 0xcd
'sswij',    # 0xce
'sswic',    # 0xcf
'sswik',    # 0xd0
'sswit',    # 0xd1
'sswip',    # 0xd2
'sswih',    # 0xd3
'ssyu',    # 0xd4
'ssyug',    # 0xd5
'ssyugg',    # 0xd6
'ssyugs',    # 0xd7
'ssyun',    # 0xd8
'ssyunj',    # 0xd9
'ssyunh',    # 0xda
'ssyud',    # 0xdb
'ssyul',    # 0xdc
'ssyulg',    # 0xdd
'ssyulm',    # 0xde
'ssyulb',    # 0xdf
'ssyuls',    # 0xe0
'ssyult',    # 0xe1
'ssyulp',    # 0xe2
'ssyulh',    # 0xe3
'ssyum',    # 0xe4
'ssyub',    # 0xe5
'ssyubs',    # 0xe6
'ssyus',    # 0xe7
'ssyuss',    # 0xe8
'ssyung',    # 0xe9
'ssyuj',    # 0xea
'ssyuc',    # 0xeb
'ssyuk',    # 0xec
'ssyut',    # 0xed
'ssyup',    # 0xee
'ssyuh',    # 0xef
'sseu',    # 0xf0
'sseug',    # 0xf1
'sseugg',    # 0xf2
'sseugs',    # 0xf3
'sseun',    # 0xf4
'sseunj',    # 0xf5
'sseunh',    # 0xf6
'sseud',    # 0xf7
'sseul',    # 0xf8
'sseulg',    # 0xf9
'sseulm',    # 0xfa
'sseulb',    # 0xfb
'sseuls',    # 0xfc
'sseult',    # 0xfd
'sseulp',    # 0xfe
'sseulh',    # 0xff
)



from qrcode import constants, exceptions, util
from qrcode.image.base import BaseImage

import six
from bisect import bisect_left


def make(data=None, **kwargs):
    qr = QRCode(**kwargs)
    qr.add_data(data)
    return qr.make_image()


def _check_version(version):
    if version < 1 or version > 40:
        raise ValueError(
            "Invalid version (was %s, expected 1 to 40)" % version)


def _check_box_size(size):
    if int(size) <= 0:
        raise ValueError(
            "Invalid box size (was %s, expected larger than 0)" % size)


def _check_mask_pattern(mask_pattern):
    if mask_pattern is None:
        return
    if not isinstance(mask_pattern, int):
        raise TypeError(
            "Invalid mask pattern (was %s, expected int)" % type(mask_pattern))
    if mask_pattern < 0 or mask_pattern > 7:
        raise ValueError(
            "Mask pattern should be in range(8) (got %s)" % mask_pattern)


class QRCode:

    def __init__(self, version=None,
                 error_correction=constants.ERROR_CORRECT_M,
                 box_size=10, border=4,
                 image_factory=None,
                 mask_pattern=None):
        _check_box_size(box_size)
        self.version = version and int(version)
        self.error_correction = int(error_correction)
        self.box_size = int(box_size)
        # Spec says border should be at least four boxes wide, but allow for
        # any (e.g. for producing printable QR codes).
        self.border = int(border)
        _check_mask_pattern(mask_pattern)
        self.mask_pattern = mask_pattern
        self.image_factory = image_factory
        if image_factory is not None:
            assert issubclass(image_factory, BaseImage)
        self.clear()

    def clear(self):
        """
        Reset the internal data.
        """
        self.modules = None
        self.modules_count = 0
        self.data_cache = None
        self.data_list = []

    def add_data(self, data, optimize=20):
        """
        Add data to this QR Code.

        :param optimize: Data will be split into multiple chunks to optimize
            the QR size by finding to more compressed modes of at least this
            length. Set to ``0`` to avoid optimizing at all.
        """
        if isinstance(data, util.QRData):
            self.data_list.append(data)
        else:
            if optimize:
                self.data_list.extend(
                    util.optimal_data_chunks(data, minimum=optimize))
            else:
                self.data_list.append(util.QRData(data))
        self.data_cache = None

    def make(self, fit=True):
        """
        Compile the data into a QR Code array.

        :param fit: If ``True`` (or if a size has not been provided), find the
            best fit for the data to avoid data overflow errors.
        """
        if fit or (self.version is None):
            self.best_fit(start=self.version)
        if self.mask_pattern is None:
            self.makeImpl(False, self.best_mask_pattern())
        else:
            self.makeImpl(False, self.mask_pattern)

    def makeImpl(self, test, mask_pattern):
        _check_version(self.version)
        self.modules_count = self.version * 4 + 17
        self.modules = [None] * self.modules_count

        for row in range(self.modules_count):

            self.modules[row] = [None] * self.modules_count

            for col in range(self.modules_count):
                self.modules[row][col] = None   # (col + row) % 3

        self.setup_position_probe_pattern(0, 0)
        self.setup_position_probe_pattern(self.modules_count - 7, 0)
        self.setup_position_probe_pattern(0, self.modules_count - 7)
        self.setup_position_adjust_pattern()
        self.setup_timing_pattern()
        self.setup_type_info(test, mask_pattern)

        if self.version >= 7:
            self.setup_type_number(test)

        if self.data_cache is None:
            self.data_cache = util.create_data(
                self.version, self.error_correction, self.data_list)
        self.map_data(self.data_cache, mask_pattern)

    def setup_position_probe_pattern(self, row, col):
        for r in range(-1, 8):

            if row + r <= -1 or self.modules_count <= row + r:
                continue

            for c in range(-1, 8):

                if col + c <= -1 or self.modules_count <= col + c:
                    continue

                if (0 <= r and r <= 6 and (c == 0 or c == 6)
                        or (0 <= c and c <= 6 and (r == 0 or r == 6))
                        or (2 <= r and r <= 4 and 2 <= c and c <= 4)):
                    self.modules[row + r][col + c] = True
                else:
                    self.modules[row + r][col + c] = False

    def best_fit(self, start=None):
        """
        Find the minimum size required to fit in the data.
        """
        if start is None:
            start = 1
        _check_version(start)

        # Corresponds to the code in util.create_data, except we don't yet know
        # version, so optimistically assume start and check later
        mode_sizes = util.mode_sizes_for_version(start)
        buffer = util.BitBuffer()
        for data in self.data_list:
            buffer.put(data.mode, 4)
            buffer.put(len(data), mode_sizes[data.mode])
            data.write(buffer)

        needed_bits = len(buffer)
        self.version = bisect_left(util.BIT_LIMIT_TABLE[self.error_correction],
                                   needed_bits, start)
        if self.version == 41:
            raise exceptions.DataOverflowError()

        # Now check whether we need more bits for the mode sizes, recursing if
        # our guess was too low
        if mode_sizes is not util.mode_sizes_for_version(self.version):
            self.best_fit(start=self.version)
        return self.version

    def best_mask_pattern(self):
        """
        Find the most efficient mask pattern.
        """
        min_lost_point = 0
        pattern = 0

        for i in range(8):
            self.makeImpl(True, i)

            lost_point = util.lost_point(self.modules)

            if i == 0 or min_lost_point > lost_point:
                min_lost_point = lost_point
                pattern = i

        return pattern

    def print_tty(self, out=None):
        """
        Output the QR Code only using TTY colors.

        If the data has not been compiled yet, make it first.
        """
        if out is None:
            import sys
            out = sys.stdout

        if not out.isatty():
            raise OSError("Not a tty")

        if self.data_cache is None:
            self.make()

        modcount = self.modules_count
        out.write("\x1b[1;47m" + (" " * (modcount * 2 + 4)) + "\x1b[0m\n")
        for r in range(modcount):
            out.write("\x1b[1;47m  \x1b[40m")
            for c in range(modcount):
                if self.modules[r][c]:
                    out.write("  ")
                else:
                    out.write("\x1b[1;47m  \x1b[40m")
            out.write("\x1b[1;47m  \x1b[0m\n")
        out.write("\x1b[1;47m" + (" " * (modcount * 2 + 4)) + "\x1b[0m\n")
        out.flush()

    def print_ascii(self, out=None, tty=False, invert=False):
        """
        Output the QR Code using ASCII characters.

        :param tty: use fixed TTY color codes (forces invert=True)
        :param invert: invert the ASCII characters (solid <-> transparent)
        """
        if out is None:
            import sys
            if sys.version_info < (2, 7):
                # On Python versions 2.6 and earlier, stdout tries to encode
                # strings using ASCII rather than stdout.encoding, so use this
                # workaround.
                import codecs
                out = codecs.getwriter(sys.stdout.encoding)(sys.stdout)
            else:
                out = sys.stdout

        if tty and not out.isatty():
            raise OSError("Not a tty")

        if self.data_cache is None:
            self.make()

        modcount = self.modules_count
        codes = [six.int2byte(code).decode('cp437')
                 for code in (255, 223, 220, 219)]
        if tty:
            invert = True
        if invert:
            codes.reverse()

        def get_module(x, y):
            if (invert and self.border and
                    max(x, y) >= modcount+self.border):
                return 1
            if min(x, y) < 0 or max(x, y) >= modcount:
                return 0
            return self.modules[x][y]

        for r in range(-self.border, modcount+self.border, 2):
            if tty:
                if not invert or r < modcount+self.border-1:
                    out.write('\x1b[48;5;232m')   # Background black
                out.write('\x1b[38;5;255m')   # Foreground white
            for c in range(-self.border, modcount+self.border):
                pos = get_module(r, c) + (get_module(r+1, c) << 1)
                out.write(codes[pos])
            if tty:
                out.write('\x1b[0m')
            out.write('\n')
        out.flush()

    def make_image(self, image_factory=None, **kwargs):
        """
        Make an image from the QR Code data.

        If the data has not been compiled yet, make it first.
        """
        _check_box_size(self.box_size)
        if self.data_cache is None:
            self.make()

        if image_factory is not None:
            assert issubclass(image_factory, BaseImage)
        else:
            image_factory = self.image_factory
            if image_factory is None:
                # Use PIL by default
                from qrcode.image.pil import PilImage
                image_factory = PilImage

        im = image_factory(
            self.border, self.modules_count, self.box_size, **kwargs)
        for r in range(self.modules_count):
            for c in range(self.modules_count):
                if self.modules[r][c]:
                    im.drawrect(r, c)
        return im

    def setup_timing_pattern(self):
        for r in range(8, self.modules_count - 8):
            if self.modules[r][6] is not None:
                continue
            self.modules[r][6] = (r % 2 == 0)

        for c in range(8, self.modules_count - 8):
            if self.modules[6][c] is not None:
                continue
            self.modules[6][c] = (c % 2 == 0)

    def setup_position_adjust_pattern(self):
        pos = util.pattern_position(self.version)

        for i in range(len(pos)):

            for j in range(len(pos)):

                row = pos[i]
                col = pos[j]

                if self.modules[row][col] is not None:
                    continue

                for r in range(-2, 3):

                    for c in range(-2, 3):

                        if (r == -2 or r == 2 or c == -2 or c == 2 or
                                (r == 0 and c == 0)):
                            self.modules[row + r][col + c] = True
                        else:
                            self.modules[row + r][col + c] = False

    def setup_type_number(self, test):
        bits = util.BCH_type_number(self.version)

        for i in range(18):
            mod = (not test and ((bits >> i) & 1) == 1)
            self.modules[i // 3][i % 3 + self.modules_count - 8 - 3] = mod

        for i in range(18):
            mod = (not test and ((bits >> i) & 1) == 1)
            self.modules[i % 3 + self.modules_count - 8 - 3][i // 3] = mod

    def setup_type_info(self, test, mask_pattern):
        data = (self.error_correction << 3) | mask_pattern
        bits = util.BCH_type_info(data)

        # vertical
        for i in range(15):

            mod = (not test and ((bits >> i) & 1) == 1)

            if i < 6:
                self.modules[i][8] = mod
            elif i < 8:
                self.modules[i + 1][8] = mod
            else:
                self.modules[self.modules_count - 15 + i][8] = mod

        # horizontal
        for i in range(15):

            mod = (not test and ((bits >> i) & 1) == 1)

            if i < 8:
                self.modules[8][self.modules_count - i - 1] = mod
            elif i < 9:
                self.modules[8][15 - i - 1 + 1] = mod
            else:
                self.modules[8][15 - i - 1] = mod

        # fixed module
        self.modules[self.modules_count - 8][8] = (not test)

    def map_data(self, data, mask_pattern):
        inc = -1
        row = self.modules_count - 1
        bitIndex = 7
        byteIndex = 0

        mask_func = util.mask_func(mask_pattern)

        data_len = len(data)

        for col in six.moves.xrange(self.modules_count - 1, 0, -2):

            if col <= 6:
                col -= 1

            col_range = (col, col-1)

            while True:

                for c in col_range:

                    if self.modules[row][c] is None:

                        dark = False

                        if byteIndex < data_len:
                            dark = (((data[byteIndex] >> bitIndex) & 1) == 1)

                        if mask_func(row, c):
                            dark = not dark

                        self.modules[row][c] = dark
                        bitIndex -= 1

                        if bitIndex == -1:
                            byteIndex += 1
                            bitIndex = 7

                row += inc

                if row < 0 or self.modules_count <= row:
                    row -= inc
                    inc = -inc
                    break

    def get_matrix(self):
        """
        Return the QR Code as a multidimensonal array, including the border.

        To return the array without a border, set ``self.border`` to 0 first.
        """
        if self.data_cache is None:
            self.make()

        if not self.border:
            return self.modules

        width = len(self.modules) + self.border*2
        code = [[False]*width] * self.border
        x_border = [False]*self.border
        for module in self.modules:
            code.append(x_border + module + x_border)
        code += [[False]*width] * self.border

        return code



from application.model import (
    OperationDescription,
    BenefitAppliers,
)
from application.model.configuration import Configurations
from application.model.repository.benefit_applier import DefaultBenefitApplierRepository
from application.repository.pos import DBRepository
from typing import Optional, Dict


class UnlockBenefitInteractor(object):

    def __init__(self, configs, db_repository, benefit_appliers_repositories):
        # type: (Configurations, DBRepository, Dict[BenefitAppliers: DefaultBenefitApplierRepository]) -> None

        self.configs = configs
        self.logger = self.configs.logger
        self.db_repository = db_repository
        self.benefit_appliers_repositories = benefit_appliers_repositories

    def unlock_benefit(self, benefit_id, originator_pos_id, benefit_applier, retry_quantity=0):
        # type: (str, int, BenefitAppliers, Optional[int]) -> None

        success = False
        try:
            message = "Unlocking benefit. BenefitId: {}; OriginatorPosId: {}; BenefitApplier: {}; Try: {}"
            self.logger.info(message.format(benefit_id, originator_pos_id, benefit_applier, retry_quantity))

            if benefit_applier not in [None, 'None', '']:
                self.benefit_appliers_repositories[benefit_applier].unlock_benefit(
                    benefit_id=benefit_id,
                    pos_id=originator_pos_id,
                )
            success = True
        except Exception as _:
            self.logger.exception("Error unlocking benefit: {}".format(benefit_id))
        finally:
            self.db_repository.manage_operation_status(success,
                                                       benefit_id,
                                                       originator_pos_id,
                                                       OperationDescription.UNLOCKED,
                                                       benefit_applier,
                                                       retry_quantity)



from ._SitefFunction import SitefFunctions, CancelCodes, ResultCodes, CommandCodes, FieldCodes, FinalizeActionFunction

__all__ = ["SitefFunctions", "CancelCodes", "ResultCodes", "CommandCodes", "FieldCodes", "FinalizeActionFunction"]



# encoding: utf-8
from escposformatter import DefaultEscPosFormatter
from mwposreport import ReporterFactory
from msgbus import MBEasyContext

from reportsdata.deliveryreport import DeliveryGenerator, DeliveryPresenter
from deliveryreport.repository import DeliveryOrderRepository, DeliveryProductRepository


class ReportGenerator(object):
    @staticmethod
    def generate(
        mb_context,             # type: MBEasyContext
        pos_id,                 # type: str
        order_id,               # type: str
        pos_list,               # type: str
        show_customer_phone,    # type: str
    ):
        # type: (...) -> str

        if not pos_list:
            pos_list = [pos_id]
        else:
            pos_list = pos_list.split(",")

        order_repository = DeliveryOrderRepository(mb_context, pos_list)
        product_repository = DeliveryProductRepository(mb_context)
        report = ReporterFactory.build_reporter(
            pos_id=pos_id,
            generator=DeliveryGenerator(
                order_repository=order_repository,
                product_repository=product_repository,
                order_id=order_id,
                show_customer_phone=show_customer_phone.lower() == "true",
            ),
            presenter=DeliveryPresenter(56),
        ).create_base_report()
        return DefaultEscPosFormatter().format(report.get_parts())



# -*- coding: utf-8 -*-

import locale

from datetime import datetime
from typing import List, Union

from cashreport import CashReport
from cashreport.repository import (
    AccountRepository,
    FiscalRepository,
    OrderRepository,
    PosCtrlRepository,
    ProductRepository,
    TableRepository,
    TenderRepository,
)
from msgbus import MBEasyContext

from utils import str_float, get_pos_list

DOUBLE_LINE_SEPARATOR = u"======================================\n"


class SalesSummary(object):
    TYPE_BUSINESS_PERIOD = "BusinessPeriod"

    def __init__(self,
                 mb_context,    # type: MBEasyContext
                 store_id,      # type: Union[str, None]
                 ):
        # type: (...) -> None

        self.mb_context = mb_context
        self.store_id = store_id

        self.order_repository = OrderRepository(mb_context, get_pos_list(mb_context), '')
        self.account_repository = AccountRepository(mb_context)
        self.tender_repository = TenderRepository(mb_context)
        self.fiscal_repository = FiscalRepository(mb_context)
        self.pos_ctrl_repository = PosCtrlRepository(mb_context)
        self.table_repository = TableRepository(mb_context)
        self.product_repository = ProductRepository(mb_context)
        self.cash_report = CashReport(
            self.order_repository,
            self.account_repository,
            self.tender_repository,
            self.fiscal_repository,
            self.pos_ctrl_repository,
            self.table_repository,
            self.product_repository,
            None,
        )

        try:
            locale.setlocale(locale.LC_ALL, "portuguese_brazil")
        except Exception:
            locale.setlocale(locale.LC_ALL, "pt_BR.UTF-8")

    def generate_summary_report(
        self,
        pos_id,         # type: int
        initial_date,   # type: datetime
        end_date,       # type: datetime
    ):
        # type: (...) -> str

        paid_orders = self.order_repository.get_paid_orders_by_business_period(
            initial_date=initial_date,
            end_date=end_date,
            operator_id=None,
            report_pos=None,
        )

        opened_orders = self.order_repository.get_opened_orders(
            report_type=self.TYPE_BUSINESS_PERIOD,
            initial_date=initial_date,
            end_date=end_date,
            operator_id=None
        )

        header = self._build_header(initial_date, end_date)

        paid_orders = filter(lambda x: x != "error", paid_orders)
        tables_by_sector = self.table_repository.get_tables_by_sector(paid_orders)
        paid_orders = self.cash_report.fill_orders_sector(tables_by_sector, paid_orders)
        grouped_paid_orders_by_sector = self.cash_report.get_grouped_paid_orders_by_sector(paid_orders)
        body, additional_info = self._build_body(pos_id, paid_orders, opened_orders, grouped_paid_orders_by_sector)

        average_info = self.table_repository.get_sales_average_info(initial_date, end_date, "")
        table_info = self.cash_report.build_table_info(average_info, grouped_paid_orders_by_sector, additional_info)
        body += table_info

        report = header + body

        return report.encode("utf-8")

    def _build_header(
        self,
        initial_date,     # type: datetime
        end_date          # type: datetime
    ):
        # type: (...) -> unicode

        pos_list = "Todos"
        report_name = "Resumo de Vendas"
        operator_name = "Todos"

        header = DOUBLE_LINE_SEPARATOR
        header += u"{0}".format(report_name).center(38)
        header += u"\nLoja.........: " + self.store_id.upper()
        header += u"\nData/hora....: " + datetime.now().strftime("%d/%m/%Y %H:%M:%S") + "\n"
        formatted_initial_date = initial_date.strftime("%d/%m/%y")
        formatted_end_date = end_date.strftime("%d/%m/%y")
        header += u"{0:.<13}: {1} - {2}\n".format("Periodo", formatted_initial_date, formatted_end_date)
        header += u"Operador.....: " + u"{0}\n".format(operator_name[:20])
        temp_pos_list = u"POS incluidos: " + u"{0}".format(pos_list)
        header += temp_pos_list[:38]
        if len(temp_pos_list) > 38:
            header += (u"\n                %s" % temp_pos_list[38:60])
        if len(temp_pos_list) > 66:
            header += (u"\n                %s" % temp_pos_list[60:82])
        header += u"\n"
        header += DOUBLE_LINE_SEPARATOR

        return header

    def _build_body(
        self,
        pos_id,                              # type: str
        paid_orders,                         # type: List[object]
        opened_orders,                       # type: List[str]
        grouped_paid_orders_by_sector=None,  # type: dict
    ):
        # type: (...) -> (unicode, List[dict])

        (
            qty_paid_orders,
            qty_discount,
            qty_donation,
            qty_tip,
            value_paid_orders,
            value_discount,
            value_donation,
            value_tip
        ) = self.cash_report.get_paid_order_values(paid_orders)

        qty_opened_orders, value_opened_orders = self.cash_report.get_opened_orders_values(pos_id, opened_orders)

        delivery_paid_orders = filter(lambda x: x.sale_type == 3 and x.delivery_originator, paid_orders)
        qty_delivery_orders, value_delivery_orders = self.cash_report.get_delivery_orders_values(delivery_paid_orders)

        manual_delivery_paid_orders = filter(lambda x: x.sale_type == 3 and not x.delivery_originator, paid_orders)
        (
            qty_manual_delivery_orders,
            value_manual_delivery_orders
        ) = self.cash_report.get_delivery_orders_values(manual_delivery_paid_orders)

        total_quantity_delivery = qty_delivery_orders + qty_manual_delivery_orders
        total_value_delivery = value_delivery_orders + value_manual_delivery_orders

        body = u"Descricao          Qtd    Total\n"
        body += DOUBLE_LINE_SEPARATOR
        sales_without_tip = str_float(value_paid_orders + value_opened_orders)
        body += u"Venda s/ Gorjetas: [{0:>4}] R${1:>10}\n".format(qty_paid_orders, sales_without_tip)

        total_quantity_table = 0
        total_value_table = 0.0
        lines = []
        for sector in sorted(grouped_paid_orders_by_sector, key=lambda x: grouped_paid_orders_by_sector[x]['value']):
            quantity = grouped_paid_orders_by_sector[sector]['quantity']
            value = str_float(grouped_paid_orders_by_sector[sector]['value'])
            total_quantity_table += quantity
            total_value_table += grouped_paid_orders_by_sector[sector]['value']
            lines.append(u"    {0:.<13}: [{1:>4}] R${2:>10}\n".format(sector, quantity, value))

        total_bill_paid = value_paid_orders + value_tip
        total_quantity_counter = qty_paid_orders - total_quantity_table - total_quantity_delivery
        total_value_counter = total_bill_paid - total_value_table - total_value_delivery

        additional_info = [
            {
                "name": "Balcao",
                "value": total_value_counter / total_quantity_counter if total_quantity_counter > 0 else 0,
                "quantity": total_quantity_counter
            },
            {
                "name": "Delivery",
                "value": total_value_delivery / total_quantity_delivery if total_quantity_delivery > 0 else 0,
                "quantity": total_quantity_delivery
            }
        ]

        return body, additional_info



from typing import List
from model import OrderData


class OrderDataFormatter(object):
    @staticmethod
    def format_order_data(order_data):
        # type: (OrderData) -> unicode
        return str(order_data.order_id) + "\0" + \
               str(order_data.pos_id) + "\0" + \
               order_data.order_picture + "\0" + \
               order_data.order_xml

    def format_order_data_list(self, order_data_list):
        # type: (List[OrderData]) -> unicode
        ret = ""
        for order_data in order_data_list:
            ret += self.format_order_data(order_data) + "\0"

        return ret[0:-1]



# encoding: utf-8

import locale

from datetime import datetime
from typing import (
    List,
    Union,
)

from basereport import SimpleReport, ReportColumnDefinition, TableReport, AlignTypes
from report import Presenter, Part
from report.command import NewLineCommand, AlignCommand, RepeatCommand, FontCommand
from reportsdata.benefitsreport.dto import BenefitsBodyDto
from reportsdata.benefitsreport.builders import BenefitConsolidatedBuilder
from utils.report_formatters import (
    format_line,
    remove_accents,
    str_float,
    set_inline_text,
)

from systools import sys_log_exception


class BenefitsPresenter(Presenter):
    def __init__(self, columns):
        # type: (int) -> None

        self.columns = columns
        self.header_column_definition = [
            ReportColumnDefinition(width=16, after_fill_text=': ', fill_with_char='.'),
            ReportColumnDefinition(width=columns - 16)
        ]

        self.header_table_column_definitions = [
            ReportColumnDefinition(width=7, after_text=' ', fill_with_char='', align=AlignTypes.LEFT),
            ReportColumnDefinition(width=12, after_text=' ', fill_with_char=''),
            ReportColumnDefinition(width=4, after_text=' ', fill_with_char='', align=AlignTypes.CENTER),
            ReportColumnDefinition(width=8),
            ReportColumnDefinition(width=7),
        ]

        self.sub_header_table_column_definitions = [
            ReportColumnDefinition(width=7),
            ReportColumnDefinition(width=12, after_text='', fill_with_char=''),
            ReportColumnDefinition(width=4, after_text='', fill_with_char='', align=AlignTypes.CENTER),
            ReportColumnDefinition(width=7, align=AlignTypes.RIGHT),
            ReportColumnDefinition(width=8, align=AlignTypes.RIGHT),
        ]

        self.items_table_column_definitions = [
            ReportColumnDefinition(
                width=17,
                after_fill_text=': ',
                fill_with_char='.',
            ),
            ReportColumnDefinition(
                width=columns - 17,
                fill_with_char='',
                align=AlignTypes.RIGHT,
            ),
        ]

        self.header_table_column_consolidated_definitions = [
            ReportColumnDefinition(width=16, align=AlignTypes.CENTER, fill_with_char=''),
            ReportColumnDefinition(width=4, align=AlignTypes.CENTER, fill_with_char=''),
            ReportColumnDefinition(width=18, align=AlignTypes.CENTER, fill_with_char=''),
            ReportColumnDefinition(width=18, align=AlignTypes.CENTER, fill_with_char=''),
        ]

        self.body_column_definition = [
            ReportColumnDefinition(
                width=20,
                after_fill_text='',
                fill_with_char='',
            ),
            ReportColumnDefinition(
                width=columns - 20,
                align=AlignTypes.RIGHT,
            ),
        ]

        self.center_definitions = [
            ReportColumnDefinition(
                width=columns,
                align=AlignTypes.CENTER,
            ),
        ]

        try:
            locale.setlocale(locale.LC_ALL, "portuguese_brazil")
        except:  # noqa:
            locale.setlocale(locale.LC_ALL, "pt_BR.UTF-8")

    def present(self, dto):
        try:
            parts = [Part(commands=[FontCommand(FontCommand.Font.B)])]
            parts.extend(self._build_header(dto.header))
            parts.extend(self._build_items(dto.body))
            parts.extend([Part(None, [NewLineCommand(), NewLineCommand()])])
        except Exception as _:
            sys_log_exception("Error building Benefits Report")
            raise

        return SimpleReport(parts, self.columns)

    def _build_header(self, header):
        parts = [
            Part(None, [NewLineCommand()]),
            self._get_title(header),
            Part(None, [NewLineCommand(), NewLineCommand()])
        ]
        parts.extend(self._get_promotion_name(header))
        parts.extend(self._get_store_name(header))
        parts.extend(self._get_date_time())
        parts.extend(self._get_period(header))
        parts.extend(self._get_operator_name(header))
        parts.extend([Part(None, [RepeatCommand(self.columns, "="), NewLineCommand()])])

        return parts

    def _build_items(
            self,
            body    # type: BenefitsBodyDto
    ):
        # type: (...) -> List

        builder_method = "_build_items_{}".format(body.type_report)
        data = getattr(self, builder_method)(body)
        return data

    def _build_items_consolidated(
            self,
            body    # type: BenefitsBodyDto
    ):
        # type: (...) -> List

        builder = BenefitConsolidatedBuilder(body, self.columns)
        return builder.generate()

    def _build_items_detailed(
            self,
            body    # type: BenefitsBodyDto
    ):
        # type: (...) -> List

        data = []
        keys_size = len(body.items.keys())

        for period, applied_benefits in body.items.items():
            data.extend(self._get_detailed_period_header(period))
            keys_size -= 1
            items_size = len(applied_benefits)
            for benefit in applied_benefits:
                items_size -= 1
                self.set_row_body(
                    data,
                    title="$ORDER",
                    text=str(benefit["order_id"]).zfill(5),
                )
                self.set_row_body(
                    data,
                    title="$HOUR",
                    text=benefit["order_time"],
                )
                self.set_row_body(
                    data,
                    title="$OPERATOR",
                    text=str(benefit["operator"]),
                )
                self.set_row_body(
                    data,
                    title="$TYPE",
                    text=benefit["discount_type"],
                )
                self.set_row_body(
                    data,
                    title="$AMOUNT_DISCOUNT_BODY_REPORT",
                    text=str_float(benefit["benefit_total_value"]),
                )
                self.set_row_body(
                    data,
                    title="$AMOUNT_ORDER_BODY_REPORT",
                    text=str_float(benefit["total_gross"]),
                )
                if items_size > 0:
                    data.extend(self._get_divider(divider="-"))

            if keys_size > 0:
                data.extend(self._get_divider())

        return data

    def set_row_body(
        self,
        parts,      # type: List
        title,      # type: str
        text,       # type: str
    ):
        set_inline_text(
            parts=parts,
            title=str(title),
            text=str(text),
            columns=self.columns,
            definition=self.items_table_column_definitions
        )

    def _get_promotion_name(self, header):
        promotion_name = header.promotion_name
        title = "$BENEFIT_REPORT_PROMOTION_NAME"
        return format_line(title, promotion_name, self.columns, self.header_column_definition)

    def _get_store_name(self, header):
        return format_line("$STORE", header.store_name, self.columns, self.header_column_definition)

    def _get_date_time(self):
        # type: (...) -> List[TableReport]

        current_datetime = datetime.now().strftime("%x %X")
        return format_line("$BENEFIT_REPORT_DATE_TIME", current_datetime, self.columns, self.header_column_definition)

    def _get_period(self, header):
        period = "{0} - {1}".format(header.start_date, header.end_date)
        return format_line("$BENEFIT_REPORT_PERIOD", period, self.columns, self.header_column_definition)

    def _get_operator_name(self, header):
        operator = remove_accents(header.operator).upper()
        return format_line("$OPERATOR", operator, self.columns, self.header_column_definition)

    def _get_divider(
        self,
        divider="=",     # type: str
    ):
        # type: (...) -> List[Part]

        return [Part(None, [RepeatCommand(self.columns, divider), NewLineCommand()])]

    def _get_detailed_period_header(
            self,
            period,  # type: str
    ):
        # type: (...) -> List[Union[Part, TableReport]]

        period_date = datetime.strptime(period, "%Y%m%d")

        header = []
        header.extend(
            format_line(
                None,
                period_date.strftime('%x'),
                self.columns,
                self.center_definitions
            )
        )

        header.extend(self._get_divider())

        return header

    @staticmethod
    def _get_title(header):
        return Part(header.title, [AlignCommand(AlignCommand.Alignment.center)])

    @staticmethod
    def _get_new_line():
        # type: (...) -> List[Part]

        return [Part(None, [NewLineCommand()])]



from messagebus import TokenPriority, TokenCreator


class LoyaltyToken(object):
    MESSAGE_GROUP = "111"

    @staticmethod
    def create_token(
        number,  # type: str
    ):
        # type: (...) -> int

        return TokenCreator.create_token(
            priority=TokenPriority.low,
            message_group=LoyaltyToken.MESSAGE_GROUP,
            number=number,
        )



import json
from datetime import datetime
from logging import Logger
from memorycache import CacheManager
from string import upper

import msgbus
from mbcontextmessagehandler import MbContextMessageBus
from typing import (
    Any,
    Dict,
    Optional,
)

from messagebus import (
    DefaultToken,
    Message,
)
from model.config import Config
from model.error_type import ExternalServicesErrorsType
from model.exception import ExternalServiceErrors
from model.repository import PosRepositoryModel
from interactor.route.store.get_status.dto import (
    GetStoreResponse,
    GetStoreRequest,
    StatusResponse,
)
from model.tokens import BusTokens
from validation_util import validate_api_key

FISCAL_KEY = "fiscal"
PRINTER_KEY = "printer"
MANAGER_KEY = "manager"
MENU_KEY = "menu"
EFT_KEY = "eft"


class GetStoreInteractor(object):
    def __init__(
        self,
        logger,                 # type: Logger
        config,                 # type: Config
        pos_repository,         # type: PosRepositoryModel
        message_bus,            # type: MbContextMessageBus
        memory_cache,           # type: Dict[str, Any[Dict[str, CacheManager], CacheManager]]
    ):
        # type: (...) -> None

        self.logger = logger
        self.config = config
        self.pos_repository = pos_repository
        self.message_bus = message_bus
        self.memory_cache = memory_cache

    def execute(
        self,
        request,    # type: GetStoreRequest
    ):
        # type: (...) -> GetStoreResponse

        validate_api_key(request, self.config.api_key_mapping)

        menu_id = self.memory_cache[MENU_KEY].get_cached_object(new_object_func=self.pos_repository.get_catalog_version)
        if not menu_id:
            raise ExternalServiceErrors(ExternalServicesErrorsType.UnableToGetMenuId)

        fiscal = self.memory_cache[FISCAL_KEY].get_cached_object(new_object_func=self._fiscal_status)

        pos_id = str(request.pos_id)
        printer = self.memory_cache[PRINTER_KEY][pos_id].get_cached_object(self._get_printer_status, False, pos_id)
        eft = self.memory_cache[EFT_KEY][pos_id].get_cached_object(self._get_eft_status, False, pos_id)
        manager = self.memory_cache[MANAGER_KEY][pos_id].get_cached_object(self._get_manager_status, False, pos_id)

        return GetStoreResponse(
            menu_id=menu_id,
            printer=printer,
            fiscal=fiscal,
            eft=eft,
            manager=manager,
        )

    def _fiscal_status(
        self,
    ):
        # type: (...) -> StatusResponse

        status_fiscal = False
        if self.config.nf_type == 'NFCE':
            status_fiscal = self.pos_repository.get_sefaz_status()
        elif self.config.nf_type == 'SAT':
            status_fiscal = self.pos_repository.get_store_sat_status()

        return StatusResponse(
            status=status_fiscal,
            message="OK" if status_fiscal else "Erro Fiscal",
        )

    def _get_printer_status(
        self,
        pos_id,     # type: int
    ):
        # type: (...) -> StatusResponse

        try:
            device_name = self._get_device_manager_info(
                pos_id=pos_id,
                device_type="printer",
            )
            if device_name is None:
                raise Exception("Printer unavailable")

            message_printer = Message(
                token=msgbus.TK_PRN_GETSTATUS,
                data_type=msgbus.FM_PARAM,
                data=None,
            )
            msg_printer = self.message_bus.send_message(
                component=device_name,
                message=message_printer,
            )

            if msg_printer.token == DefaultToken.TK_SYS_ACK and msg_printer.data == "0":
                return StatusResponse(
                    status=True,
                    message="OK",
                )

            return StatusResponse(
                status=False,
                message="NOK",
            )
        except Exception:
            return StatusResponse(
                status=False,
                message="NOK",
            )

    def _get_eft_status(
        self,
        pos_id,     # type: int
    ):
        # type: (...) -> StatusResponse

        try:
            device_name = self._get_device_manager_info(
                pos_id=pos_id,
                device_type="eft",
            )
            if device_name is None:
                raise Exception("Eft unavailable")

            data = "{};{};{}".format(pos_id, datetime.now().strftime("%Y%m%d"), datetime.now().strftime('%H:%M:%S'))
            message = Message(
                token=BusTokens.TK_ELECTRONIC_PAYMENT_CONNECTIVITY_TEST,
                data_type=msgbus.FM_PARAM,
                data=data,
            )
            msg = self.message_bus.send_message(
                component=device_name,
                message=message,
            )

            if msg.token == DefaultToken.TK_SYS_ACK and upper(msg.data) == "TRUE":
                return StatusResponse(
                    status=True,
                    message="OK",
                )

            return StatusResponse(
                status=False,
                message="NOK",
            )

        except Exception:
            return StatusResponse(
                status=False,
                message="NOK",
            )

    def _get_manager_status(
        self,
        pos_id,     # type: int
    ):
        # type: (...) -> StatusResponse

        totem_was_closed = self.pos_repository.get_manager_status(pos_id=pos_id)
        model = self.pos_repository.get_model(pos_id=pos_id)
        message = self.pos_repository.translate_message(
            model=model,
            message="CLOSED" if totem_was_closed else "OPENED",
        )
        return StatusResponse(
            status=not totem_was_closed,
            message=message,
        )

    def _get_device_manager_info(
        self,
        pos_id,         # type: int
        device_type,    # type: str
    ):
        # type: (...) -> Optional[str]

        request = json.dumps(
            {
                "posId": pos_id,
                "deviceType": device_type
            }
        )
        message = Message(
            token=BusTokens.TK_GET_USED_SERVICE,
            data_type=msgbus.FM_PARAM,
            data=request,
        )
        msg = self.message_bus.send_message(
            component="DeviceManager",
            message=message,
        )

        if msg.token == DefaultToken.TK_SYS_ACK:
            return msg.data

        return None




# -*- coding: utf-8 -*-

from __future__ import unicode_literals

"""barcode.errors
"""
__docformat__ = 'restructuredtext en'


class BarcodeError(Exception):
    def __init__(self, msg):
        self.msg = msg

    def __str__(self):
        return self.msg


class IllegalCharacterError(BarcodeError):
    """Raised when a barcode-string contains illegal characters."""


class BarcodeNotFoundError(BarcodeError):
    """Raised when an unknown barcode is requested."""


class NumberOfDigitsError(BarcodeError):
    """Raised when the number of digits do not match."""


class WrongCountryCodeError(BarcodeError):
    """Raised when a JAN (Japan Article Number) don't starts with 450-459
    or 490-499.
    """



from typing import Union, Optional, List, Dict
from logging import Logger

from mwrepository import ProductRepository
from production.box import ProductionBox
from production.box._OrderChangerProductionBox import OrderChangerProductionBox


class SorterBox(OrderChangerProductionBox):
    def __init__(self, name, sons, product_repository, sort_order, logger=None):
        # type: (str, Optional[Union[List[ProductionBox]], ProductionBox], ProductRepository, Dict, Logger) -> None

        super(SorterBox, self).__init__(name, sons, logger)
        self.sort_products = {line: product_repository.get_part_codes_of_jit_lines([line]) for line in sort_order}
        self.sort_order = sort_order

    def change_order(self, order):
        order_items = []

        if order.items:
            for jit, value in sorted(self.sort_order.items(), key=lambda x: x[1]):
                items_to_append = [item for item in order.items if item.part_code in self.sort_products[jit]]
                items_to_append.sort(key=lambda x: x.description)
                order_items.extend(items_to_append)

            if len(order_items) != len(order.items):
                order_items.extend([item for item in order.items if item not in order_items])

        order.items = order_items

        return order



from ._AsyncUuidMessageProcessor import AsyncUuidMessageProcessor
from ._BaseThread import BaseThread
from ._BenefitAppliers import BenefitAppliers
from ._ListenedEvents import ListenedEvents
from ._ListenedTokens import ListenedTokens
from ._NotMappedEvents import NotMappedEvents
from ._OperationDescription import OperationDescription
from ._OperationStatus import OperationStatus
from ._OperationDescription import OperationDescription
from ._BenefitControllerDto import BenefitControllerDto
from ._SentTokens import NDiscountTokens



from abc import ABCMeta, abstractmethod
from typing import Optional  # noqa


class ScreenActions(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def show_listbox(self, posid, options, message='$SELECT_AN_OPTION', title='', defvalue='', buttons='$OK|$CANCEL',
                     icon='', timeout=60000, asynch=False):
        # type: (str, List[str], Optional[str], Optional[str], Optional[str], Optional[str],  Optional[str], Optional[int], Optional[bool]) -> None
        raise NotImplementedError()



import json

from application.processor.add_benefit.dto import AddBenefitValidationErrorCode
from typing import Optional

from application.interactor import AddBenefitToApplyInteractor
from application.model import Configurations
from application.model.exceptions import (
    UserLevelNotAllowedException,
)
from application.model.transactions import AddBenefitToApplyUnitOfWork
from application.service import OrderService, BenefitService
from messagebus import (
    DataType,
    DefaultToken,
    Event,
    Message,
    MessageBus,
    NakMessage,
)
from messageprocessorutil import UuidMessageProcessor


class AddBenefitProcessor(UuidMessageProcessor):
    def __init__(self, configs, interactor, order_service, benefit_service):
        # type: (Configurations, AddBenefitToApplyInteractor, OrderService, BenefitService) -> None

        super(AddBenefitProcessor, self).__init__()
        self.configs = configs
        self.interactor = interactor
        self.order_service = order_service
        self.benefit_service = benefit_service
        self.logger = configs.logger

    def parse_data(self, data, event_type):
        # type: (str, str) -> AddBenefitToApplyUnitOfWork

        parsed_data = json.loads(data)
        pos_id = int(parsed_data.get("posId"))
        raw_benefit = parsed_data.get("benefit", {})
        benefit_applier = parsed_data.get("benefit_applier")
        tender_id = parsed_data.get("tender_id")
        user_auth_level = parsed_data.get("user_auth_level")

        return self.benefit_service.get_unit_of_work(
                pos_id,
                raw_benefit,
                benefit_applier=benefit_applier,
                tender_id=tender_id,
                user_auth_level=user_auth_level
        )

    def call_business(self, uow):
        # type: (AddBenefitToApplyUnitOfWork) -> None

        self.interactor.apply(uow)

    def format_response(self, message_bus, message, event, request, response):
        message_bus.reply_message(message, Message(DefaultToken.TK_SYS_ACK, DataType.string, ""))

    def format_exception(
        self,           # type: AddBenefitProcessor
        message_bus,    # type: MessageBus
        message,        # type: Optional[Message]
        event,          # type: Optional[Event]
        input_obj,      # type: AddBenefitToApplyUnitOfWork
        exception,      # type: Exception
    ):
        # type: (...) -> None

        if isinstance(exception, UserLevelNotAllowedException):
            error_code = AddBenefitValidationErrorCode.NDISCOUNT_ERROR_USER_MIN_LEVEL.name
            message_bus.reply_message(
                message=message,
                reply_message=NakMessage(
                    data=error_code + "\0" + str(exception.min_level_required),
                    data_type=DataType.param,
                ),
            )
        else:
            message_bus.reply_message(
                message=message,
                reply_message=NakMessage(
                    data=str(exception),
                    data_type=DataType.param,
                ),
            )

    def format_parse_exception(self, message_bus, message, event, data, exception):
        message_bus.reply_message(message, Message(DefaultToken.TK_SYS_NAK, DataType.param, str(exception)))



import datetime
from xml.etree import cElementTree as eTree

import sysactions

from printerconstants import (
    TXT_ALIGN_CT,
    CTL_LF,
    TXT_ALIGN_LT,
    SEPARATOR,
    TXT_BOLD_OFF,
    TXT_BOLD_ON,
    TXT_SIZE,
    TXT_WIDTH,
    TXT_HEIGHT,
    TXT_NORMAL,
    HW_INIT,
    TXT_FONT_A,
    COLS,
)


class CashPaymentReport(object):

    def __init__(
        self,
        pos_id,      # type: int
    ):
        # type: (...) -> None

        self.pos_id = pos_id

    def generate(
        self,
        model,              # type: eTree
        current_order,      # type: eTree

    ):
        # type: (...) -> str

        header = self._get_header(
            model=model,
        )

        body = self._get_body(
            model=model,
            current_order=current_order,
        )

        footer = self._get_footer(
            model=model,
        )

        report = "{}{}{}".format(
            header,
            body,
            footer,
        )

        return report.encode(
            encoding="utf-8",
        )

    @staticmethod
    def _get_header(
        model,      # type: eTree
    ):
        # type: (...) -> str

        store_name = sysactions.get_storewide_config(
            parameter="Store.RazaoSocial",
        )

        text = "{}{}{}{}{}".format(
            TXT_FONT_A,
            TXT_ALIGN_CT,
            store_name,
            CTL_LF,
            TXT_FONT_A,
        )

        current_date = datetime.datetime.now().strftime(
            "%d/%m/%Y %H:%M:%S",
        )

        label_date = sysactions.translate_message(
            model=model,
            message_key="DATE_TIME",
        )

        text += "{0}: {1}".format(
            label_date,
            current_date,
        )

        text += "{}{}{}{}".format(
            CTL_LF,
            SEPARATOR,
            CTL_LF,
            TXT_ALIGN_LT,
        )

        return text

    def _get_body(
        self,
        model,              # type: eTree
        current_order,      # type: eTree
    ):
        # type: (...) -> str

        label_totem = self._get_terminal_value(
            model=model,
        )

        label_value = self._get_order_value(
            model=model,
            current_order=current_order,
        )

        size_space_between = COLS - (len(label_totem) + len(label_value))

        text = label_totem
        text += (" " * size_space_between)
        text += label_value

        text += "{}{}{}{}{}{}".format(
            CTL_LF,
            CTL_LF,
            TXT_ALIGN_LT,
            TXT_ALIGN_CT,
            TXT_BOLD_ON,
            TXT_SIZE + chr(TXT_WIDTH[2] + TXT_HEIGHT[2]),
        )

        label_order_number = sysactions.translate_message(
            model=model,
            message_key="ORDER",
        )
        order_number = current_order.get("orderId").zfill(3)
        order_number = order_number[3:] if len(order_number) > 3 else order_number

        text += "{0}: {1}".format(
            label_order_number,
            order_number,
        )

        return text

    def _get_terminal_value(
        self,
        model,      # type: eTree
    ):
        # type: (...) -> str

        label_totem = sysactions.translate_message(
            model=model,
            message_key="TOTEM_TERMINAL",
        )

        return "{0}: {1}".format(
            label_totem,
            self.pos_id,
        )

    @staticmethod
    def _get_order_value(
        model,              # type: eTree
        current_order,      # type: eTree
    ):
        # type: (...) -> str

        label_value = sysactions.translate_message(
            model=model,
            message_key="VALUE",
        )

        return "{0:>6}: R${1:>8}".format(
            label_value,
            current_order.get("totalGross"),
        )

    @staticmethod
    def _get_footer(
        model,      # type: eTree
    ):
        # type: (...) -> str

        label_message = sysactions.translate_message(
            model=model,
            message_key="GO_TO_THE_CASHIER",
        )
        label_message = label_message.replace(
            "\\n",
            CTL_LF,
        )
        text = u"{}{}{}{}{}".format(
            TXT_BOLD_OFF,
            CTL_LF,
            CTL_LF,
            TXT_NORMAL,
            label_message,
        )

        return text

    @staticmethod
    def _rollback_printer_default_font(
    ):
        # type: (...) -> str

        return "{}{}{}{}".format(
            HW_INIT,
            TXT_NORMAL,
            TXT_BOLD_OFF,
            TXT_ALIGN_LT,
        )



# encoding: utf-8
from benefitsreport import BenefitsReport
from report import Generator
from reportsdata.benefitsreport.dto import BenefitsGeneratorDto, BenefitsHeaderDto, BenefitsBodyDto


class BenefitsGenerator(Generator):
    def __init__(self, order_repository, initial_date, end_date, operator_id, benefits):

        self.order_repository = order_repository
        self.initial_date = initial_date
        self.end_date = end_date
        self.operator_id = operator_id
        self.benefits = benefits

    def generate_data(self):
        # type: (...) -> BenefitsGeneratorDto

        benefits_report = BenefitsReport(self.order_repository)
        if self.benefits and self.benefits[0].get("id") is not None:
            generator_dto = self.detailed_report(benefits_report)
        else:
            generator_dto = self.consolidated_report(benefits_report)

        return generator_dto

    def detailed_report(self, benefits_report):
        # type: (BenefitsReport) -> BenefitsGeneratorDto

        orders = benefits_report.get_benefits_orders_detailed(
                self.initial_date,
                self.end_date,
                self.operator_id,
                self.benefits
        )
        promotion_name = self.benefits[0].get("name").encode("utf8")
        title = "$BENEFIT_REPORT_DETAILED"
        header = BenefitsHeaderDto(title, promotion_name, self.operator_id, self.initial_date, self.end_date)
        body = BenefitsBodyDto(orders, "detailed")

        return BenefitsGeneratorDto(header, body)

    def consolidated_report(self, benefits_report):
        # type: (BenefitsReport) -> BenefitsGeneratorDto

        orders = benefits_report.get_benefits_orders_consolidated(self.initial_date,
                                                                  self.end_date,
                                                                  self.operator_id)
        promotion_name = "$ALL"
        title = "$BENEFIT_REPORT_CONSOLIDATED"
        header = BenefitsHeaderDto(title, promotion_name, self.operator_id, self.initial_date, self.end_date)
        body = BenefitsBodyDto(orders, "consolidated")

        return BenefitsGeneratorDto(header, body)



from logging import Logger
from mw_helper import convert_to_dict, ensure_list
from production.box import ProductionBox
from production.box._OrderChangerProductionBox import OrderChangerProductionBox
from production.manager import OrderChanger
from production.model import ProductionOrder
from typing import Optional, List, Union


class OrderPropertiesFilterBox(OrderChangerProductionBox):
    def __init__(
            self,
            name,                       # type: str
            sons,                       # type: Union[List[ProductionBox], ProductionBox]
            order_changer,              # type: OrderChanger
            order_property,             # type: str
            allowed_values=None,        # type: Optional[List[str]]
            excluded_values=None,       # type: Optional[List[str]]
            remove_order=True,          # type: bool
            tags_to_add=None,           # type: Optional[List[str]]
            logger=None                 # type: Optional[Logger]
    ):
        # type: (...) -> None

        super(OrderPropertiesFilterBox, self).__init__(name, sons, logger)
        self.order_changer = order_changer
        self.order_property = order_property
        self.allowed_values = convert_to_dict(ensure_list(allowed_values))
        self.excluded_values = convert_to_dict(ensure_list(excluded_values))
        self.remove_order = remove_order
        self.tags_to_add = ensure_list(tags_to_add)

    def change_order(
            self,
            order   # type: ProductionOrder
    ):
        # type: (...) -> Optional[ProductionOrder]

        if not self.order_property:
            return order

        order_property = str(getattr(order, self.order_property))
        is_excluded = len(self.excluded_values) > 0 and order_property in self.excluded_values
        is_not_allowed = len(self.allowed_values) > 0 and order_property not in self.allowed_values
        if is_excluded or is_not_allowed:
            self._tag_items(order)

            if self.remove_order:
                return None
            else:
                order.items = []

        return order

    def _tag_items(
        self,
        order,  # type: ProductionOrder
    ):
        # type: (...) -> None

        items_to_tag = []
        tags_to_add = self.tags_to_add

        for item in order.items:
            items_to_tag.append(item)

        if len(items_to_tag) == 0:
            return

        def _inner_tag_items(
            root_order,     # type: ProductionOrder
        ):
            # type: (...) -> bool

            some_item_tagged = False
            for item_to_tag in items_to_tag:
                root_item = root_order.get_item_by_line_id(item_to_tag.get_line_id())
                for tag in tags_to_add:
                    root_item.add_tag(tag, item_to_tag.displayed_jit)
                    some_item_tagged = True

            return some_item_tagged

        self.order_changer.change_order(order.order_id, _inner_tag_items)



import sys
try:
    import unittest2 as unittest
except ImportError:
    import unittest
try:
    from unittest import mock
except ImportError:
    import mock

from qrcode.console_scripts import main


def bad_read():
    raise UnicodeDecodeError('utf-8', b'0x80', 0, 1, 'invalid start byte')


class ScriptTest(unittest.TestCase):

    @mock.patch('os.isatty', lambda *args: True)
    @mock.patch('qrcode.main.QRCode.print_ascii')
    def test_isatty(self, mock_print_ascii):
        main(['testtext'])
        mock_print_ascii.assert_called_with(tty=True)

    @mock.patch('os.isatty', lambda *args: False)
    @mock.patch('sys.stdout')
    def test_piped(self, mock_stdout):
        main(['testtext'])

    @mock.patch('os.isatty', lambda *args: True)
    @mock.patch('qrcode.main.QRCode.print_ascii')
    def test_stdin(self, mock_print_ascii):
        mock_stdin = mock.Mock(sys.stdin)
        stdin_buffer = getattr(mock_stdin, 'buffer', mock_stdin)
        stdin_buffer.read.return_value = 'testtext'
        with mock.patch('sys.stdin', mock_stdin):
            main([])
        self.assertTrue(stdin_buffer.read.called)
        mock_print_ascii.assert_called_with(tty=True)

    @unittest.skipIf(sys.version_info[0] < 3, 'Python 3')
    @mock.patch('os.isatty', lambda *args: True)
    @mock.patch('qrcode.main.QRCode.print_ascii')
    def test_stdin_py3_unicodedecodeerror(self, mock_print_ascii):
        mock_stdin = mock.Mock(sys.stdin)
        mock_stdin.buffer.read.return_value = 'testtext'
        mock_stdin.read.side_effect = bad_read
        with mock.patch('sys.stdin', mock_stdin):
            # sys.stdin.read() will raise an error...
            self.assertRaises(UnicodeDecodeError, sys.stdin.read)
            # ... but it won't be used now.
            main([])
        mock_print_ascii.assert_called_with(tty=True)

    @mock.patch('os.isatty', lambda *args: True)
    @mock.patch('qrcode.main.QRCode.print_ascii')
    def test_optimize(self, mock_print_ascii):
        main('testtext --optimize 0'.split())

    @mock.patch('sys.stdout')
    def test_factory(self, mock_stdout):
        main('testtext --factory svg'.split())

    @mock.patch('sys.stderr')
    def test_bad_factory(self, mock_stderr):
        self.assertRaises(SystemExit, main, 'testtext --factory fish'.split())



import json
from xml.etree import cElementTree as eTree

from application.model import NDiscountTokens
from application.model.repository.benefit_applier import DefaultBenefitApplierRepository
from messagebus import Message, DataType
from msgbus import TK_SYS_ACK


class NDiscountCouponApplierRepository(DefaultBenefitApplierRepository):

    def __init__(self, configs, message_bus):
        super(NDiscountCouponApplierRepository, self).__init__(configs, message_bus)

        self.benefits = None

    def check_coupon_id(
        self,       # type: NDiscountCouponApplierRepository
        pos_id,     # type: int
        coupon_id,  # type: str
    ):
        # type: (...) -> bool

        self._get_all_benefits_cache(pos_id)
        return self.benefits and coupon_id in self.benefits

    def get_benefit_and_lock(
        self,       # type: NDiscountCouponApplierRepository
        coupon_id,  # type: str
    ):
        # type: (...) -> None

        return self.benefits[coupon_id]

    def unlock_benefit(
        self,
        benefit_id,     # type: str
        pos_id,         # type: int
    ):
        # type: (...) -> None

        return

    def burn_benefit(
        self,
        benefit_id,     # type: str
        pos_id,         # type: int
    ):
        # type: (...) -> None

        return

    def get_benefit_info(self, data):
        # type: (str) -> None

        return

    def get_benefit_id_by_custom_property(self, order_picture):
        # type: (eTree) -> None

        return

    def _get_all_benefits_cache(self, pos_id):
        if self.benefits is None:
            data = json.dumps(dict(posId=pos_id))
            message = Message(NDiscountTokens.TK_NDISCOUNT_GET_BENEFITS, DataType.string, data)
            response = self.message_bus.send_message("nDiscount", message)
            if response.token == TK_SYS_ACK:
                benefits = json.loads(response.data)

                self.benefits = {}
                for benefit in benefits:
                    self.benefits[benefit.get("benefitId")] = benefit.get("benefitJson")




from unittest import TestCase
import textwrap

import simplejson as json
from simplejson.compat import StringIO

class TestIndent(TestCase):
    def test_indent(self):
        h = [['blorpie'], ['whoops'], [], 'd-shtaeou', 'd-nthiouh',
             'i-vhbjkhnth',
             {'nifty': 87}, {'field': 'yes', 'morefield': False} ]

        expect = textwrap.dedent("""\
        [
        \t[
        \t\t"blorpie"
        \t],
        \t[
        \t\t"whoops"
        \t],
        \t[],
        \t"d-shtaeou",
        \t"d-nthiouh",
        \t"i-vhbjkhnth",
        \t{
        \t\t"nifty": 87
        \t},
        \t{
        \t\t"field": "yes",
        \t\t"morefield": false
        \t}
        ]""")


        d1 = json.dumps(h)
        d2 = json.dumps(h, indent='\t', sort_keys=True, separators=(',', ': '))
        d3 = json.dumps(h, indent='  ', sort_keys=True, separators=(',', ': '))
        d4 = json.dumps(h, indent=2, sort_keys=True, separators=(',', ': '))

        h1 = json.loads(d1)
        h2 = json.loads(d2)
        h3 = json.loads(d3)
        h4 = json.loads(d4)

        self.assertEqual(h1, h)
        self.assertEqual(h2, h)
        self.assertEqual(h3, h)
        self.assertEqual(h4, h)
        self.assertEqual(d3, expect.replace('\t', '  '))
        self.assertEqual(d4, expect.replace('\t', '  '))
        # NOTE: Python 2.4 textwrap.dedent converts tabs to spaces,
        #       so the following is expected to fail. Python 2.4 is not a
        #       supported platform in simplejson 2.1.0+.
        self.assertEqual(d2, expect)

    def test_indent0(self):
        h = {3: 1}
        def check(indent, expected):
            d1 = json.dumps(h, indent=indent)
            self.assertEqual(d1, expected)

            sio = StringIO()
            json.dump(h, sio, indent=indent)
            self.assertEqual(sio.getvalue(), expected)

        # indent=0 should emit newlines
        check(0, '{\n"3": 1\n}')
        # indent=None is more compact
        check(None, '{"3": 1}')

    def test_separators(self):
        lst = [1,2,3,4]
        expect = '[\n1,\n2,\n3,\n4\n]'
        expect_spaces = '[\n1, \n2, \n3, \n4\n]'
        # Ensure that separators still works
        self.assertEqual(
            expect_spaces,
            json.dumps(lst, indent=0, separators=(', ', ': ')))
        # Force the new defaults
        self.assertEqual(
            expect,
            json.dumps(lst, indent=0, separators=(',', ': ')))
        # Added in 2.1.4
        self.assertEqual(
            expect,
            json.dumps(lst, indent=0))



from application.model.configuration import Configurations
from application.repository.api import APIRepository


class UnlockVoucherInteractor(object):
    def __init__(self, configs, api_repository):
        # type: (Configurations, APIRepository) -> None

        self.logger = configs.logger
        self.configs = configs
        self.api_repository = api_repository

    def unlock_benefit(
        self,           # type: UnlockVoucherInteractor
        benefit_id,     # type: str
        pos_id,         # type: int
    ):
        # type: (...) -> None

        try:
            self.logger.info("Unlocking benefitId: {}".format(benefit_id))
            self.api_repository.unlock_benefit(
                benefit_id=benefit_id,
                pos_id=pos_id,
            )
            self.logger.info("Benefit successfully unlocked")
        except Exception as ex:
            self.logger.error("Error at unlock_benefit", exc_info=ex)
            raise ex



from ._LoyaltyToken import LoyaltyToken
from messagebus import TokenCreator, TokenPriority


class LoyaltyResponseToken(LoyaltyToken):
    TK_LOYALTY_ERROR = TokenCreator.create_token(
        priority=TokenPriority.low,
        message_group=LoyaltyToken.LOYALTY_MESSAGE_GROUP,
        number="F002"
    )



import json
from json import dumps, loads
from xml.etree import cElementTree as eTree

from application.model.benefit import OrderBenefitInfo, Benefit, Conditions
from posot import OrderTakerException, OrderTaker
from sysactions import get_model
from typing import Dict, Optional, List

from ._SaleLine import SaleLine
from application.util import get_range_discount_ids


class Order(object):
    def __init__(self, order_picture, sale_lines, model):
        pos_id = int(order_picture.get("posId"))

        self.pos_id = pos_id
        self.pod_type = order_picture.get("podType")
        self.id = int(order_picture.get("orderId")) if order_picture.get("orderId") else None
        self.order_state = order_picture.get("state")
        self.sale_lines = [SaleLine(o) for o in sale_lines]
        self.total_amount = float(order_picture.get("totalAmount", 0)) + float(order_picture.get("taxTotal", 0))
        self.total_gross = float(order_picture.get("totalGross", self.total_amount))
        self.discount_amount = float(order_picture.get("discountAmount", 0))
        self.due_amount = float(order_picture.attrib["dueAmount"])
        self.sale_type = order_picture.get("saleTypeDescr")
        self.order_picture = order_picture
        self.custom_properties = self._get_custom_order_properties(order_picture)
        self.sector = self.custom_properties.get("SECTOR")
        self.table_id = self.custom_properties.get("TABLE_ID") if self.custom_properties.get("TABLE_ID") else None
        self.brand = self.custom_properties.get("BRAND") if self.custom_properties.get("BRAND") else None

        self.seats = 0
        self.tip = 0
        self.special_catalog = None
        self.model = model
        self.get_table_info()

        self.applied_benefits = self._get_applied_benefits(order_picture)

    def get_table_info(self):
        model = self.model
        if model is None:
            model = get_model(self.pos_id)
        table_xml = self._get_table_xml(model)

        if table_xml:
            self.sector = table_xml.get("sector", None)
            self.seats = int(table_xml.get("serviceSeats", 0))
            self.tip = float(table_xml.get("tip", 0))
            self._get_table_custom_property_info(table_xml)

    def _get_table_custom_property_info(self, table_xml):
        for custom_property in table_xml.findall(".//Properties"):
            service_period = custom_property.get('SERVICE_PERIOD_TABLE', None)
            if service_period:
                self.special_catalog = loads(service_period)
                break

    def _get_table_xml(self, model):
        # type: (eTree.XML) -> Optional[eTree.XML]

        from tablemgrapi import get_pos_ts

        table_id = self.table_id
        if not table_id:
            return

        pos_ts = get_pos_ts(model)
        return eTree.XML(pos_ts.getTableInfo(table_id))

    @staticmethod
    def _get_custom_order_properties(order_picture):
        # type: (eTree.XML) -> Dict[str, str]

        custom_properties = {}
        for custom_property in order_picture.find(".//CustomOrderProperties"):
            custom_properties[custom_property.get("key")] = custom_property.get("value")

        return custom_properties

    @staticmethod
    def get_burnt_items(order):
        # type: (Order) -> dict

        burnt_items = dict()
        for benefit in order.applied_benefits:
            for entry in benefit.burnt_items.items():
                if burnt_items.get(entry[0]) is None:
                    burnt_items[entry[0]] = entry[1]
                else:
                    burnt_items[entry[0]] = (burnt_items.get(entry[0]) + entry[1])

        return burnt_items

    @staticmethod
    def get_used_discount_ids(pos_ot):
        # type: (OrderTaker) -> List[int]
        try:
            order_discount_list = eTree.XML(pos_ot.getOrderDiscounts(applied=True)).findall(".//OrderDiscount")
            return [int(el.get("discountId")) for el in order_discount_list if el.tag == "OrderDiscount"]
        except OrderTakerException:
            return []

    @staticmethod
    def get_available_discount_ids(pos_ot):
        # type: (OrderTaker) -> set

        available_discount_ids = get_range_discount_ids()
        discount_ids = set(Order.get_used_discount_ids(pos_ot))
        return available_discount_ids.difference(discount_ids)

    @staticmethod
    def get_required_items(order, benefit):
        # type: (Order, Benefit) -> List

        conditions = benefit.conditions
        required_items_on_sale = {}

        for item in conditions.required_items_on_sale:
            key = "{};{}".format(item.context if item.context else 'any', str(item.item_id))
            required_items_on_sale[key] = item.quantity

        sale_lines = order.sale_lines
        required_items = []
        for sale_line in sale_lines:
            context = sale_line.context
            for item_index in required_items_on_sale.keys():
                required_item_context, required_item_id = item_index.split(';')

                if int(sale_line.item_id) == int(required_item_id) \
                        and (required_item_context == 'any' or required_item_context in context):
                    required_items.append(sale_line.key)

        return required_items

    @staticmethod
    def _check_and_burn_by_required_items(items_to_burn, required_items, sale_line, burnt_items, conditions):
        # type: (Dict, Dict, SaleLine, Dict, Conditions) -> None

        context = sale_line.context
        for item_index in required_items.keys():
            required_item_context, required_item_id = item_index.split(';')

            if int(sale_line.item_id) != int(required_item_id) or required_items[item_index] <= 0 \
                    or burnt_items.get(sale_line.key, -1) >= sale_line.qty:
                continue

            if required_item_context == 'any' or required_item_context in context:
                burnt_qty = min(required_items[item_index], sale_line.qty)
                required_items[item_index] -= burnt_qty
                qty = 0 if conditions.ignore_burned_items else int(burnt_items.get(sale_line.key, 0)) + burnt_qty
                items_to_burn[sale_line.key] = qty

    @staticmethod
    def _check_and_burn_by_required_groups(items_to_burn, required_groups, product_groups, sale_line, burnt_items,
                                           conditions):
        # type: (Dict, Dict, Dict, SaleLine, Dict, Conditions) -> None

        for group_index in required_groups.keys():
            item_id = int(sale_line.item_id)
            if item_id not in product_groups[group_index] or required_groups[group_index] <= 0 \
                    or burnt_items.get(sale_line.key, -1) >= sale_line.qty \
                    or items_to_burn.get(sale_line.key, -1) >= sale_line.qty:
                continue

            burnt_qty = min(required_groups[group_index], sale_line.qty)
            required_groups[group_index] -= burnt_qty
            qty = 0
            if not conditions.ignore_burned_items:
                qty = int(items_to_burn.get(sale_line.key, 0)) + int(burnt_items.get(sale_line.key, 0)) + burnt_qty
            items_to_burn[sale_line.key] = qty

    def get_items_to_burn(self, order, benefit, burnt_items, product_groups):
        # type: (Order, Benefit, Dict, Dict) -> Dict

        if not benefit.conditions.burn_items:
            return {}

        sale_lines = order.sale_lines
        conditions = benefit.conditions
        items_to_burn = {}

        required_items_on_sale = {}
        required_groups_on_sale = {}

        for item in conditions.required_items_on_sale:
            key = "{};{}".format(item.context if item.context else 'any', str(item.item_id))
            required_items_on_sale[key] = item.quantity

        for group in conditions.required_groups_on_sale:
            required_groups_on_sale[group.group] = group.quantity

        for sale_line in sale_lines:
            self._check_and_burn_by_required_items(
                items_to_burn, required_items_on_sale, sale_line, burnt_items, conditions
            )
            self._check_and_burn_by_required_groups(
                items_to_burn, required_groups_on_sale, product_groups, sale_line, burnt_items, conditions
            )

        for key in items_to_burn.keys():
            if items_to_burn.get(key, 0) == 0:
                items_to_burn.pop(key)

        return items_to_burn

    def get_applied_benefits_from_order_picture(self, order_picture):
        # type: (Optional[eTree]) -> List

        benefit_list = self.get_benefit_list_from_order_picture(order_picture)
        if not benefit_list:
            return []

        return json.loads(benefit_list.get("benefitList", "[]"))

    @staticmethod
    def get_benefit_list_from_order_picture(order_picture):
        # type: (Optional[eTree]) -> Dict

        try:
            value = order_picture.find(".//OrderProperty[@key=\"BENEFIT_LIST\"]").get("value")
            return json.loads(value)
        except OrderTakerException:
            pass
        except AttributeError:
            pass
        return {}

    def _get_applied_benefits(self, order_picture):
        order_picture_benefits = self.get_applied_benefits_from_order_picture(order_picture)
        applied_benefits = [
            OrderBenefitInfo(
                    benefit.get("benefit"),
                    benefit.get("used_discount_ids"),
                    benefit.get("added_sale_lines"),
                    benefit.get("burnt_items"),
                    benefit.get("promotion_tender"),
                    benefit.get("discount_item_values"),
                    benefit.get("payment_discount_id")
            )
            for benefit in order_picture_benefits
        ]

        return applied_benefits

    def to_json(self):
        return dumps(self, default=lambda o: o.__dict__ if hasattr(o, '__dict__') else str(o), sort_keys=True)



import json

from logging import Logger
from typing import (
    Any,
    Optional,
    Union,
)

from helper import remove_accents
from posot import OrderTaker

from model.config import Config
from model.repository import PosRepositoryModel
from model.services import (
    CouponServiceModel,
    OrderServiceModel
)
from model.error_type import (
    NCouponErrorResponse,
    NotFoundErrorsType,
    PosValidationErrorsType,
)
from model.exception import (
    AuthValidationErrors,
    BlockedError,
    ClientCustomError,
    InUseError,
    NotFoundError,
    OperationErrors,
    ValidationError,
)
from model.response import Responses
from interactor.route.order.apply_coupon.dto import ApplyCouponRequest
from validation_util import check_if_pos_id_is_valid, validate_api_key, check_if_order_is_in_progress_or_totaled


FREE_COUPON_DATA = 'FREE_COUPON_DATA'


class ApplyCouponInteractor(object):
    def __init__(
            self,
            logger,             # type: Logger
            config,             # type: Config
            pos_repository,     # type: PosRepositoryModel
            coupon_service,     # type: CouponServiceModel
            order_service       # type: OrderServiceModel
    ):
        # type: (...) -> None

        self.logger = logger
        self.config = config
        self.pos_repository = pos_repository
        self.coupon_service = coupon_service
        self.order_service = order_service
        self.model = None
        self.pos_ot = None  # type: Optional[OrderTaker]
        self.price_list = None

    def execute(self, request):
        # type: (ApplyCouponRequest) -> None

        pos_id = validate_api_key(request, self.config.api_key_mapping)
        check_if_pos_id_is_valid(self.pos_repository, pos_id)

        coupon_data, coupon_applied = self.pos_repository.apply_coupon(
            pos_id=pos_id,
            coupon_code=request.coupon_code
        )

        self.model = self.pos_repository.get_model(pos_id)
        self.pos_ot = self.pos_repository.get_pos_ot(self.model)
        self.price_list = self.pos_repository.get_price_list(self.model)

        if self.pos_repository.has_current_order(self.model):
            check_if_order_is_in_progress_or_totaled(self.pos_repository, self.model)

        if coupon_applied:
            self.logger.info("Success coupon applied. CouponData: {}".format(coupon_data))
            order = self.pos_repository.get_current_order(self.model)
            order_id = order.get('orderId')
            order = self.pos_repository.get_order_details(
                pos_id=pos_id,
                order_id=order_id,
                pos_ot=self.pos_ot
            )
            if order:
                coupon_data = self.order_service.convert_mw_order_to_order_detail(order)

            return coupon_data

        else:
            self._raise_custom_error(
                coupon_data=coupon_data,
                language=request.language[:2] if request.language else "pt",
            )

    @staticmethod
    def _format_error_message(
        data,           # type: json
        language,       # type: str
        error_code,     # type: int
    ):
        # type: (...) -> Optional[str, dict]

        if data in (None, "None", ""):
            return

        try:
            data = json.loads(data)
            if error_code != 406:
                return data.get("message") or None

            message = data.get("message")
            translated_message = message[language] if message is not None and language in message.keys() else ""

            return remove_accents(text=translated_message)
        except Exception as _:
            return data

    @staticmethod
    def _get_error_code_message(
        message,    # type: str
    ):
        # type: (...) -> int

        if message == NCouponErrorResponse.COUPON_NOT_FOUND.value:
            return 404
        if message == NCouponErrorResponse.CUSTOM_ERROR.value:
            return 406
        if message == NCouponErrorResponse.COUPON_LOCKED.value:
            return 409
        if message == NCouponErrorResponse.COUPON_ALREADY_USED.value:
            return 410
        return 422

    def _raise_custom_error(
        self,
        coupon_data,    # type: Any[Union[str, json]]
        language,       # type: str
    ):
        # type: (...) -> None

        error_code = ""
        if '\0' in coupon_data:
            error_code, coupon_data = coupon_data.split('\0')
            status_code = self._get_error_code_message(message=error_code)
        else:
            status_code = self._get_error_code_message(message=coupon_data)

        message = self._format_error_message(
            data=coupon_data,
            language=language.lower(),
            error_code=status_code,
        )
        if not message:
            message = error_code

        self.logger.info("Error applying coupon. CouponData: {}".format(message))
        if status_code == Responses.AuthValidation:
            raise AuthValidationErrors(PosValidationErrorsType.CannotApplyCoupon)

        if status_code == Responses.NotFoundError:
            raise NotFoundError(
                NotFoundErrorsType.CouponNotFound,
                message,
            )
        if status_code == Responses.ClientCustomError:
            raise ClientCustomError(
                PosValidationErrorsType.CannotApplyCouponByPartnerRules,
                message,
            )
        if status_code == Responses.BlockedError:
            raise BlockedError(
                PosValidationErrorsType.CouponBlockedByAnotherRequest,
                message,
            )
        if status_code == Responses.InUseError:
            raise InUseError(
                PosValidationErrorsType.AlreadyUsedCoupon,
                message,
            )
        if status_code == Responses.OperationErrors:
            raise OperationErrors(PosValidationErrorsType.CannotApplyCoupon)

        raise ValidationError(
            PosValidationErrorsType.CannotApplyCoupon,
            message,
        )




from production.model import CommandNotFoundException
from production.treebuilder import ViewConfiguration
from ._CommonViewBuilder import get_logger
from production.view import CommandView


class CommandViewBuilder(object):
    def __init__(self, command_processor, loggers):
        self.command_processor = command_processor
        self.loggers = loggers

    def build(self, config):
        # type: (ViewConfiguration) -> CommandView

        if "Command" not in config.extra_config:
            raise CommandNotFoundException("CommandView without command: {}".format(config.name))

        command = config.extra_config.get("Command", "")
        return CommandView(config.name, self.command_processor, command, get_logger(config, self.loggers))



# -*- coding: utf-8 -*-
#
# Copyright 2009-2017 Wander Lairson Costa
# Copyright 2009-2020 PyUSB contributors
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions of source code must retain the above copyright
# notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
# notice, this list of conditions and the following disclaimer in the
# documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the copyright holder nor the names of its
# contributors may be used to endorse or promote products derived from
# this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import sys

__all__ = ['AutoFinalizedObject']


class _AutoFinalizedObjectBase(object):
    """
    Base class for objects that get automatically
    finalized on delete or at exit.
    """

    def _finalize_object(self):
        """Actually finalizes the object (frees allocated resources etc.).

        Returns: None

        Derived classes should implement this.
        """
        pass

    def __new__(cls, *args, **kwargs):
        """Creates a new object instance and adds the private finalizer
        attributes to it.

        Returns: new object instance

        Arguments:
        * *args, **kwargs -- ignored
        """
        instance = super(_AutoFinalizedObjectBase, cls).__new__(cls)
        instance._finalize_called = False
        return instance

    def _do_finalize_object(self):
        """Helper method that finalizes the object if not already done.

        Returns: None
        """
        if not self._finalize_called: # race-free?
            self._finalize_called = True
            self._finalize_object()

    def finalize(self):
        """Finalizes the object if not already done.

        Returns: None
        """
        # this is the "public" finalize method
        raise NotImplementedError(
            "finalize() must be implemented by AutoFinalizedObject."
        )

    def __del__(self):
        self.finalize()


if sys.hexversion >= 0x3040000:
    # python >= 3.4: use weakref.finalize
    import weakref

    def _do_finalize_object_ref(obj_ref):
        """Helper function for weakref.finalize() that dereferences a weakref
        to an object and calls its _do_finalize_object() method if the object
        is still alive. Does nothing otherwise.

        Returns: None (implicit)

        Arguments:
        * obj_ref -- weakref to an object
        """
        obj = obj_ref()
        if obj is not None:
            # else object disappeared
            obj._do_finalize_object()


    class AutoFinalizedObject(_AutoFinalizedObjectBase):

        def __new__(cls, *args, **kwargs):
            """Creates a new object instance and adds the private finalizer
            attributes to it.

            Returns: new object instance

            Arguments:
            * *args, **kwargs -- passed to the parent instance creator
                                 (which ignores them)
            """
            # Note:   Do not pass a (hard) reference to instance to the
            #         finalizer as func/args/kwargs, it'd keep the object
            #         alive until the program terminates.
            #         A weak reference is fine.
            #
            # Note 2: When using weakrefs and not calling finalize() in
            #         __del__, the object may already have disappeared
            #         when weakref.finalize() kicks in.
            #         Make sure that _finalizer() gets called,
            #         i.e. keep __del__() from the base class.
            #
            # Note 3: the _finalize_called attribute is (probably) useless
            #         for this class
            instance = super(AutoFinalizedObject, cls).__new__(
                cls, *args, **kwargs
            )

            instance._finalizer = weakref.finalize(
                instance, _do_finalize_object_ref, weakref.ref(instance)
            )

            return instance

        def finalize(self):
            """Finalizes the object if not already done."""
            self._finalizer()


else:
    # python < 3.4: keep the old behavior (rely on __del__),
    #                but don't call _finalize_object() more than once

    class AutoFinalizedObject(_AutoFinalizedObjectBase):

        def finalize(self):
            """Finalizes the object if not already done."""
            self._do_finalize_object()



from logging import Logger

from production.repository import ProductRepository
from typing import List, Optional, Union

from ._PointerBox import PointerBox
from ._ProductionBox import ProductionBox


class CustomParamPointerBox(PointerBox):
    def __init__(self, name, sons, product_repository, default_points, count_merged_items, logger=None):
        # type: (str, Optional[Union[List[ProductionBox]], ProductionBox], ProductRepository, int, bool, Logger) -> None
        product_points = product_repository.get_product_points()

        super(CustomParamPointerBox, self).__init__(
            name,
            sons,
            product_points,
            default_points,
            count_merged_items,
            logger
        )



import logging
from logging import Logger

from devicemanager.model import Configs

from cfgtools import read
from helper import config_logger


class Configurator(object):

    def __init__(
        self,
        loader_path,    # type: str
    ):
        # type: (...) -> None

        self.config = Configs()
        self.config.loader_path = loader_path
        self.reader = read(loader_path)
        self.group = None
        self.logger = None

    def read_loader_configs(
        self,
    ):
        # type: (...) -> Configs
        self.configure_loggers()
        self.config.component_name = self.reader.find_value("Identification.ComponentName")
        self.group = "DeviceManager."
        sectors_config = {}
        sectors = self.reader.find_group(self.group + "Sectors")
        for sector in sectors.groups:
            sector_name = sector.name.upper()
            if sector_name not in sectors_config:
                sectors_config[sector_name] = {}
            for key in sector.keys:
                device_type = key.name.upper()
                if device_type in sectors_config[sector_name]:
                    self.logger.info("Device manager sector loader key duplicated - {}{}".format(sector.name, key.name))

                find_key = "{}Sectors.{}.{}".format(self.group, sector.name, key.name)
                values = self.reader.find_values(find_key)
                values = values[0] if device_type != "EXCLUDEDPOS" else values
                sectors_config[sector_name][device_type] = values

        self.config.sector_config = sectors_config
        return self.config

    def configure_loggers(
        self,
    ):
        # type: (...) -> None

        config_logger(
            loader_path=self.config.loader_path,
            log_name="DeviceManager",
            max_size=10485760,
            max_files=15
        )
        self.logger = logging.getLogger("DeviceManager")

    def get_loggers(
        self,
    ):
        # type: (...) -> Logger

        return self.logger

    @staticmethod
    def _is_true(
        value,  # type: str
    ):
        # type: (...) -> bool

        return value.lower() == "true"



from typing import Dict


class Customer(object):
    def __init__(
        self,
        customer,    # type: Dict
    ):
        # type: (...) -> None

        self.id = customer.get("id", None)
        self.name = customer.get("name", None)
        self.document = customer.get("document", None)



# encoding: utf-8

from datetime import datetime
from utils import get_store_id


class BenefitsHeaderDto(object):
    def __init__(self, title, promotion_name, operator, start_date, end_date):
        # type: (str, str, str, str, str) -> ()

        self.title = title
        self.store_name = get_store_id()
        self.promotion_name = promotion_name if promotion_name is not None else title
        self.operator = operator if operator is not None else "$ALL_OPERATORS"
        self.start_date = datetime.strptime(start_date, '%Y%m%d').strftime('%d/%m/%y')
        self.end_date = datetime.strptime(end_date, '%Y%m%d').strftime('%d/%m/%y')



from abc import ABCMeta, abstractmethod
from xml.etree import cElementTree as eTree

from posot import OrderTaker

from typing import Optional, Any, List


class PosRepositoryModel(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def get_model(self, pos_id):
        # type: (int) -> eTree

        raise NotImplementedError

    @abstractmethod
    def get_pos_ot(self, model):
        # type: (eTree) -> Any

        raise NotImplementedError

    @abstractmethod
    def get_price_list(self, model):
        # type: (eTree) -> str

        raise NotImplementedError

    @abstractmethod
    def is_day_opened(self, model):
        # type: (eTree) -> bool

        raise NotImplementedError

    @abstractmethod
    def is_day_blocked(self, model):
        # type: (eTree) -> bool

        raise NotImplementedError

    @abstractmethod
    def translate_message(
        self,
        model,      # type: eTree
        message,    # type: str
    ):
        # type: (...) -> str

        raise NotImplementedError

    @abstractmethod
    def is_operator_logged(self, model):
        # type: (eTree) -> bool

        raise NotImplementedError

    @abstractmethod
    def has_operator_opened(self, model):
        # type: (eTree) -> bool

        raise NotImplementedError

    @abstractmethod
    def get_current_operator(self, model):
        # type: (eTree) -> Optional[eTree]

        raise NotImplementedError

    @abstractmethod
    def has_current_order(self, model):
        # type: (eTree) -> bool

        raise NotImplementedError

    @abstractmethod
    def is_order_in_progress(self, model):
        # type: (eTree) -> bool

        raise NotImplementedError

    @abstractmethod
    def is_order_totaled(
        self,
        model,                  # type: eTree
        order_picture=None,     # type: Optional[eTree]
    ):
        # type: (...) -> bool

        raise NotImplementedError

    @abstractmethod
    def get_current_order(self, model):
        # type: (eTree) -> Optional[eTree]

        raise NotImplementedError

    @abstractmethod
    def create_order(self, pos_id, pos_ot, price_list=None, sale_type=None):
        # type: (int, Any, Optional[str], Optional[str]) -> int

        raise NotImplementedError

    @abstractmethod
    def get_order_details(self, pos_id, order_id, pos_ot):
        # type: (int, int, Any) -> eTree

        raise NotImplementedError

    @abstractmethod
    def get_order_picture(
        self,
        pos_id,         # type: str
        order_id,       # type: str
        model,          # type: eTree
        pos_ot=None,    # type: Optional[OrderTaker]
    ):
        # type: (...) -> eTree

        raise NotImplementedError

    @abstractmethod
    def add_item_to_order(
        self,
        pos_id,          # type: int
        pos_ot,          # type: Any
        price_list,      # type: str
        model,           # type: eTree
        item,            # type: Any
        sale_type=None   # type: Optional[str]
    ):
        # type: (...) -> None

        raise NotImplementedError

    @abstractmethod
    def reopen_shopping_cart(
            self,
            pos_id,     # type: int
            pos_ot      # type: Any
    ):
        # type: (...) -> int

        raise NotImplementedError

    @abstractmethod
    def order_has_tender(self, model):
        # type: (eTree) -> Optional[eTree]

        raise NotImplementedError

    def void_order(
        self,
        pos_id,         # type: int
        pos_ot,         # type: OrderTaker
        order_id="",    # type: Optional[str]
    ):
        # type: (...) -> None

        raise NotImplementedError

    def delete_line(self, pos_id, pos_ot, line_numbers):
        # type: (int, Any, List[int]) -> None

        raise NotImplementedError

    @abstractmethod
    def get_pos_list(self):
        # type: () -> tuple

        raise NotImplementedError

    @abstractmethod
    def do_tender(self, pos_id, tender_id, pos_ot, amount):
        # type: (int, int, Any, float) -> Any

        raise NotImplementedError

    def do_clear_tender(
            self,
            pos_id,      # type: str
            model,       # type: object
            tender_id    # type: int
    ):
        # type: (...) -> Any

        raise NotImplementedError

    @abstractmethod
    def add_custom_property(self, key, value, pos_ot):
        # type: (str, str, Any) -> None

        raise NotImplementedError

    @abstractmethod
    def add_custom_properties(self,  pos_ot, properties, orderid=''):
        # type: (Any, dict, str) -> None

        raise NotImplementedError

    @abstractmethod
    def update_order_properties(self, pos_ot, posid, saletype):
        # type: (Any, str, str) -> eTree

        raise NotImplementedError

    @abstractmethod
    def do_total(self, pos_id, pos_ot):
        # type: (int, Any) -> eTree

        raise NotImplementedError

    @abstractmethod
    def get_rupture_products(self):
        # type: () -> list

        raise NotImplementedError

    @abstractmethod
    def get_menu_data(self):
        # type: () -> bytearray

        raise NotImplementedError

    @abstractmethod
    def get_nf_type(self):
        # type: () -> bool

        raise NotImplementedError

    @abstractmethod
    def get_sefaz_status(self):
        # type: () -> bool

        raise NotImplementedError

    @abstractmethod
    def get_store_sat_status(
        self,
    ):
        # type: (...) -> bool

        raise NotImplementedError

    @abstractmethod
    def get_catalog_version(self):
        # type: () -> str

        raise NotImplementedError

    @abstractmethod
    def set_order_to_paid(
        self,
        pos_id,     # type: str
        order_id,   # type: str
        model,      # type: eTree
        pos_ot,     # type: OrderTaker
    ):
        # type: (...) -> Any

        raise NotImplementedError

    @abstractmethod
    def void_tenders_and_order(self, model, pos_ot):
        # type: (eTree, object) -> Any

        raise NotImplementedError

    @abstractmethod
    def logout_delivery_user(self, pos_id, operator_id):
        # type: (int, str) -> bool

        raise NotImplementedError

    @abstractmethod
    def close_business_period(self, pos_id):
        # type: (int) -> bool

        raise NotImplementedError

    @abstractmethod
    def open_business_period(self, pos_id):
        # type: (int) -> bool

        raise NotImplementedError

    @abstractmethod
    def authenticate_user(self, user_id, user_password):
        # type: (str, str) -> Any

        raise NotImplementedError

    @abstractmethod
    def open_user(self, pos_id, user_id, long_user_name):
        # type: (int, str, str) -> bool

        raise NotImplementedError

    @abstractmethod
    def login_user(self, pos_id, user_id, long_user_name):

        raise NotImplementedError

    @abstractmethod
    def get_read_scanner(self, pos_id):

        raise NotImplementedError

    @abstractmethod
    def print_text(self, pos_id, content):
        # type: (str, str) -> bool

        raise NotImplementedError

    @abstractmethod
    def get_storewide_config(self, parameter, defval, array):
        # type: (str, Optional[bool], Optional[bool]) -> list

        raise NotImplementedError

    @abstractmethod
    def apply_coupon(self, pos_id, coupon_code):
        # type: (int, str) -> (str, bool)

        raise NotImplementedError

    @abstractmethod
    def get_order_custom_properties(self, pos_ot, key, order_id=""):
        # type: (Any, str, int) -> eTree

        raise NotImplementedError

    @abstractmethod
    def get_cfg(self, pos_id, key, default):
        # type: (int, str, str) -> Any

        raise NotImplementedError

    @abstractmethod
    def change_line_quantity(self, model, pos_ot, to_change_items):
        # type: (eTree.Element, object, List) -> Any

        raise NotImplementedError

    @abstractmethod
    def remove_coupon(
            self,
            pos_id,         # type: int
            coupon_code     # type: str
    ):
        # type: (...) -> Any

        raise NotImplementedError

    @abstractmethod
    def get_text_from_i18n(self):
        # type: () -> Any

        raise NotImplementedError

    @abstractmethod
    def get_static_config(
        self,
        pos_id  # type: str
    ):
        # type: (...) -> Any

        raise NotImplementedError

    @abstractmethod
    def get_tender_types(self):
        # type: () -> Any

        raise NotImplementedError

    @abstractmethod
    def get_navigation(
            self,
            pos_id  # type: int
    ):
        # type: (...) -> Any

        raise NotImplementedError

    def do_stored_order(
        self,
        pos_id,     # type: int
        model,      # type: eTree
    ):
        # type: (...) -> None

        raise NotImplementedError

    @abstractmethod
    def get_manager_status(
        self,
        pos_id,     # type: int
    ):
        # type: (...) -> Any

        raise NotImplementedError



from _FiscalParameterController import FiscalParameterController, FiscalParameterNotFound
from _InfAdicBuilderCommon import InfAdicBuilderCommon
from _CalculateTaxesItemCommon import CalculateTaxesItemCommon
from _TipBuilderCommon import TipBuilderCommon
from _PrinterCommon import PrinterCommon

__all__ = ["FiscalParameterController",
           "FiscalParameterNotFound",
           "InfAdicBuilderCommon",
           "CalculateTaxesItemCommon",
           "TipBuilderCommon",
           "PrinterCommon"]



from typing import List

from report import Presenter, Part
from report.command import AlignCommand, RepeatCommand
from ..dto import TipReportHeaderDto


class TipReportHeaderPresenter(Presenter):
    def present(self, dto):
        # type: (TipReportHeaderDto) -> List[Part]
        report_header = [
            Part("$TIP_REPORT", [AlignCommand(AlignCommand.Alignment.center)]), a_new_line()
        ]

        report_header.extend([
                Part(
                        "$STORE : {}".format(dto.store_id),
                        [AlignCommand(AlignCommand.Alignment.left)]), a_new_line()
        ])

        report_header.extend([
            Part("$TIP_REPORT_DATE : #DATE({0}) - #DATE({1})".format(
                dto.start_date.isoformat(),
                dto.end_date.isoformat()),
                [AlignCommand(AlignCommand.Alignment.left)]), a_new_line()])

        report_header.extend([
            Part(
                "$TIP_REPORT_CURRENT_DATE : #DATETIME({})".format(dto.current_date_time.isoformat()),
                [AlignCommand(AlignCommand.Alignment.left)]), a_new_line()
        ])

        report_header.extend([Part(commands=[RepeatCommand(38, "=")]), a_new_line()])
        return report_header


def a_new_line():
    return Part("\n")



# encoding: utf-8
import time
import datetime
import collections
import cfgtools
import os

from cStringIO import StringIO

import sysactions

from dateutil import tz
from old_helper import convert_from_localtime_to_utc, convert_from_utf_to_localtime
from typing import (
    List,
    Optional
)

from hourlysalesreport.model import Order
from hourlysalesreport.repository import OrderRepository as HourlySalesOrderRepository
from model import IntervalConfiguration
from utils import (
    DATE_TIME_FMT,
    add_table_line,
    center,
    double_line_separator,
    empty_line,
    get_store_id,
    report_default_header,
    single_line_separator
)


class HourlySalesReport(object):
    def __init__(
        self,
        order_repository,               # type: HourlySalesOrderRepository
        by_created_order_time=False     # type: Optional[str]
    ):
        # type: (...) -> None

        self.order_repository = order_repository
        self.pos_list = order_repository.pos_list
        self.by_created_order_time = by_created_order_time.lower() == "true"
        self.hourly_header = ["Tipo", "Hora", "Vendas", "TC", "TM"]
        self.hourly_columns_height = [10, 6, 9, 5, 8]
        self.hourly_align = ['left', 'right', 'right', 'right', 'right']
        self.hourly_format_type = ['string', 'string', 'float', 'integer', 'float']
        self.grouped_header = ["Tipo", "Vendas", "TC", "TM"]
        self.grouped_columns_height = [16, 9, 5, 8]
        self.grouped_align = ['left', 'right', 'right', 'right']
        self.grouped_format_type = ['string', 'float', 'integer', 'float']
        self.store_id = get_store_id()

    def generate_hourly_sales_report(self, pos_id, user_id, period):
        model = sysactions.get_model(pos_id)
        report = StringIO()
        title = "Relatorio {}".format("Lancamentos por Intervalo" if self.by_created_order_time else "Vendas por Intervalo")
        current_datetime = time.strftime(DATE_TIME_FMT)
        report.write(report_default_header(title, self.pos_list, user_id, period, current_datetime, self.store_id))
        self._hourly_sales_report_body(model, user_id, period, report)

        return report.getvalue()

    def _hourly_sales_report_body(self, model, user_id, period, report):
        year = int(period[:4])
        month = int(period[4:6])
        day = int(period[6:])

        grouped_intervals, interval_in_minutes = self._get_configurations(day, month, year)
        intervals_sale_type = []

        start_date = datetime.datetime(year, month, day).replace(tzinfo=tz.tzlocal())
        start_date_utc = convert_from_localtime_to_utc(start_date)
        end_date = start_date + datetime.timedelta(days=1)
        end_date_utc = convert_from_localtime_to_utc(end_date)

        orders = self.order_repository.get_sales_by_date(
            user_id=user_id,
            start_date=start_date_utc,
            end_date=end_date_utc,
            by_created_order_time=self.by_created_order_time
        )

        report.write(center("Intervalo de " + str(interval_in_minutes) + " minutos"))
        report.write(empty_line())
        report.write(add_table_line(self.hourly_header, self.hourly_columns_height, self.hourly_align))

        self._write_hourly_lines(end_date, grouped_intervals, interval_in_minutes, intervals_sale_type, model, orders,
                                 report, start_date)

        report.write(double_line_separator())

        self._write_intervals_lines(grouped_intervals, None, intervals_sale_type, model, report)

    def _write_intervals_lines(self, grouped_intervals, interval, intervals_sale_type, model, report):
        for grouped_interval in grouped_intervals:
            report.write(center(grouped_interval.name + " [" + grouped_interval.start_time.time().strftime(
                '%H:%M') + " - " + grouped_interval.end_time.time().strftime('%H:%M') + "]"))
            report.write(add_table_line(self.grouped_header, self.grouped_columns_height, self.hourly_align))

            orders = grouped_interval.orders
            total_sold_amount, total_sold_orders = self._write_lines(interval, model, orders, report,
                                                                     intervals_sale_type, False)

            total_sold_average = 0 if total_sold_orders == 0 else total_sold_amount / total_sold_orders
            line = ["Total", total_sold_amount, total_sold_orders, total_sold_average]
            report.write(add_table_line(line, self.grouped_columns_height, self.hourly_align, self.grouped_format_type))
            report.write(single_line_separator())

    def _write_hourly_lines(self, end_date, grouped_intervals, interval_in_minutes, intervals_sale_type, model, orders,
                            report, start_date):
        intervals = self._get_intervals(
            start_date=start_date,
            end_date=end_date,
            grouped_intervals=grouped_intervals,
            interval_in_minutes=interval_in_minutes,
            orders=orders
        )
        for interval, orders in intervals.items():
            sale_types = []

            for order in orders:
                if order.sale_type not in sale_types:
                    sale_types.append(order.sale_type)
                if order.sale_type not in intervals_sale_type:
                    intervals_sale_type.append(order.sale_type)

            total_sold_amount, total_sold_orders = self._write_lines(interval, model, orders, report, sale_types, True)

            if total_sold_amount > 0:
                total_sold_average = 0 if total_sold_orders == 0 else total_sold_amount / total_sold_orders
                line = ["Total", interval, total_sold_amount, total_sold_orders, total_sold_average]
                report.write(add_table_line(line, self.hourly_columns_height, self.hourly_align, self.hourly_format_type))
                report.write(empty_line())

    def _write_lines(self, interval, model, orders, report, sale_types, hourly_lines):
        total_sold_amount = 0
        total_sold_orders = 0
        for sale_type in sale_types:
            sale_type_orders = [order for order in orders if order.sale_type == sale_type]
            sale_type_sold_amount = sum(order.amount for order in sale_type_orders)
            sale_type_sold_orders = len(sale_type_orders)
            sale_type_average = 0

            if sale_type_sold_orders != 0:
                sale_type_average = sale_type_sold_amount / sale_type_sold_orders

            total_sold_amount += sale_type_sold_amount
            total_sold_orders += sale_type_sold_orders

            if sale_type_sold_amount > 0:
                translated_sale_type = sysactions.translate_message(model, sale_type)

                if hourly_lines:
                    line = [translated_sale_type, interval, sale_type_sold_amount, sale_type_sold_orders,
                            sale_type_average]
                    report.write(add_table_line(line, self.hourly_columns_height, self.hourly_align,
                                                self.hourly_format_type))
                else:
                    line = [translated_sale_type, sale_type_sold_amount, sale_type_sold_orders, sale_type_average]
                    report.write(add_table_line(line, self.grouped_columns_height, self.hourly_align,
                                                self.grouped_format_type))

        return total_sold_amount, total_sold_orders

    def _get_intervals(
        self,
        start_date,             # type: datetime.datetime
        end_date,               # type: datetime.datetime
        grouped_intervals,      # type: List
        interval_in_minutes,    # type: int
        orders                  # type: List[Order]
    ):
        # type (...) -> collections.OrderedDict

        grouped_intervals_processed = False
        intervals = {}
        current_date = start_date
        date_fmt = "%Y-%m-%dT%H:%M:%S" if not self.by_created_order_time else "%Y-%m-%dT%H:%M:%S.%f"
        while current_date < end_date:
            next_date = current_date + datetime.timedelta(minutes=interval_in_minutes)
            interval_key = current_date.time().strftime('%H:%M')
            intervals[interval_key] = []

            for order in orders:
                date = datetime.datetime.strptime(order.timestamp, date_fmt)
                date = convert_from_utf_to_localtime(date)
                if current_date <= date < next_date:
                    intervals[interval_key].append(order)

                if not grouped_intervals_processed:
                    for interval in grouped_intervals:
                        if interval.start_time <= date < interval.end_time:
                            interval.orders.append(order)

            grouped_intervals_processed = True
            current_date += datetime.timedelta(minutes=interval_in_minutes)
        intervals = collections.OrderedDict(sorted(intervals.items()))
        return intervals

    @staticmethod
    def _get_configurations(day, month, year):
        config = cfgtools.read(os.environ["LOADERCFG"])
        interval_in_minutes = 30 if config.find_value("HourlySalesReport.IntervalInMinutes") is None \
            else int(config.find_value("HourlySalesReport.IntervalInMinutes"))
        config_intervals = config.find_values("HourlySalesReport.Intervals") or []
        grouped_intervals = []
        try:
            for config in config_intervals:
                name = config.split("|")[0]
                start_hour = int(config.split("|")[1].split(":")[0])
                start_minute = int(config.split("|")[1].split(":")[1])
                end_hour = int(config.split("|")[2].split(":")[0])
                end_minute = int(config.split("|")[2].split(":")[1])
                grouped_intervals.append(IntervalConfiguration(
                    name,
                    datetime.datetime(year, month, day, start_hour, start_minute).replace(tzinfo=tz.tzlocal()),
                    datetime.datetime(year, month, day, end_hour, end_minute).replace(tzinfo=tz.tzlocal()))
                )
        except Exception as _:
            grouped_intervals = []

        return grouped_intervals, interval_in_minutes



import logging
import time
from logging import Logger
from typing import Optional

from messagebus import (
    DataType,
    DefaultToken,
    Message,
    MessageBus,
    TokenCreator,
    TokenPriority
)
from production.view import ProductionView
from production.model import ProductionOrder

PERIPHERALS_GROUP = "6"
TK_PRN_PRINT = TokenCreator.create_token(TokenPriority.high, PERIPHERALS_GROUP, "101")

MAX_RETRY = 10

printer_logger = logging.getLogger("ProductionPrinter")


class PickListPrinter(ProductionView):
    def __init__(
        self,
        message_bus,    # type: MessageBus
        timeout,        # type: int
        printer,        # type: str
        name,           # type: str
        logger          # type: Logger
    ):
        # type: (...) -> None

        super(PickListPrinter, self).__init__(name, logger)
        self.message_bus = message_bus
        self.timeout = timeout
        self.printer = printer

    def print_report(
        self,
        text    # type: str
    ):
        # type: (...) -> bool

        for attempt in range(1, MAX_RETRY):
            try:
                printer_logger.info("Sending to printer: {} - Attempt: {}".format(self.printer, attempt))

                if attempt == 1:
                    printer_logger.info("Printer: {} - \nPrinting:\n {}".format(self.printer, text))

                message = Message(
                    token=TK_PRN_PRINT,
                    data_type=DataType.string,
                    data=text,
                    timeout=self.timeout
                )
                reply = self.message_bus.send_message(
                    component=self.printer,
                    message=message
                )
                if reply.token != DefaultToken.TK_SYS_ACK:
                    log_message = "Invalid response from Printer: {} / {} - {}"
                    printer_logger.warning(log_message.format(self.printer, reply.token, reply.data))
                else:
                    printer_logger.info("Successfully printed on Printer: {}".format(self.printer))
                    return True
            except BaseException as ex:
                message_name = type(ex).__name__
                if attempt >= MAX_RETRY - 1:
                    log_message = "Printer {} unavailable - Failed to Print Report - {}"
                    printer_logger.warning(log_message.format(self.printer, message_name))
                else:
                    log_message = "Error when trying to print on Printer {} in attempt {} - {}"
                    printer_logger.info(log_message.format(self.printer, attempt, message_name))

            time.sleep(2 * attempt)

        return False

    def handle_order(
        self,
        order,          # type: ProductionOrder
        synch=False     # type: Optional[bool]
    ):
        # type: (...) -> None

        raise NotImplementedError()

    def get_view_tags(self):
        # type: (...) -> None
        raise NotImplementedError()



class LoginSummaryBodyDto(object):
    def __init__(self, initial_value, pos_type, authorizer):
        # type: (float, unicode, unicode) -> ()
        self.initial_value = initial_value
        self.pos_type = pos_type
        self.authorizer = authorizer



class CashTransferSummaryBodyDto(object):
    def __init__(self, cash_out, value, envelope_number, authorizer):
        # type: (bool, float, int, unicode) -> ()
        self.cash_out = cash_out
        self.value = value
        self.envelope_number = envelope_number
        self.authorizer = authorizer



import glob
import os
import sqlite3
from base64 import b64encode
from datetime import datetime
from logging import Logger
from uuid import uuid4
from zlib import crc32, compressobj, MAX_WBITS, DEFLATED, Z_DEFAULT_COMPRESSION

from model.config import Config
from model.event_dispatch import EventDispatch
from model.log import LogEntry
from model.repository import LogRepositoryModel
from model.resync import ResyncRequest
from typing import List

DATABASE_NAME_TEMPLATE = """mwapp_audit_{}.log"""

TABLE_EXISTS_QUERY = "SELECT 1 FROM sqlite_master WHERE type='table' AND name=?;"

CREATE_UUID_TABLE = "CREATE TABLE IF NOT EXISTS UUID AS SELECT ? AS value;"

CREATE_AUDITLOG_TABLE = """
    CREATE TABLE IF NOT EXISTS AuditLog (
        POSId INTEGER NOT NULL,
        BusinessPeriod VARCHAR NOT NULL,
        EvtType VARCHAR NOT NULL,
        EvtTime DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP,
        OrderId INTEGER,
        CreatedAt VARCHAR,
        SessionId VARCHAR,
        Encoded INTEGER NOT NULL DEFAULT 0,
        EvtData VARCHAR
    );
"""

FETCH_ENTRY_QUERY = """
    SELECT
        _ROWID_,
        BusinessPeriod,
        EvtType,
        Encoded,
        EvtData,
        OrderId,
        CreatedAt,
        strftime('%Y-%m-%dT%H:%M:%f', EvtTime) AS EvtTime
    FROM AuditLog
    WHERE 
        (_ROWID_ > :row_id AND BusinessPeriod = :period
        OR BusinessPeriod != :period)
        AND EvtType IN ({})
    ORDER BY _ROWID_;
"""


def _get_unsigned_crc32(data, start=0xFFFFFFFF):
    return crc32(data, start)


def _table_exists(conn, table_name):
    for _ in conn.execute(TABLE_EXISTS_QUERY, (table_name,)):
        return True
    return False


class LogRepository(LogRepositoryModel):
    def __init__(self, output_path, encode, logger, config):
        # type: (str, bool, Logger, Config) -> None

        self.output_path = output_path
        self.encode = encode
        self.logger = logger
        self.conn_timeout = config.connection_timeout
        self.audit_dir = self.create_dir()

    def create_dir(self):
        # type: () -> str
        audit_dir = os.path.normpath(self.output_path.format(**os.environ))
        if not os.path.exists(audit_dir):
            os.makedirs(audit_dir)
        return audit_dir

    def insert_entry(self, log_entry):
        # type: (LogEntry) -> EventDispatch
        database_full_path = self._get_database_name_from_business_period(log_entry.business_period)

        new_database = not os.path.exists(database_full_path)

        with self._connect_database(database_full_path) as database_conn:

            if new_database or self._database_needs_initialization(database_conn):
                uuid = uuid4().hex
                checksum = _get_unsigned_crc32(uuid)
                self.logger.debug("\nDatabase '{}' must be initialized".format(database_full_path))
                self._initialize_database(checksum, database_conn, uuid)
            else:
                self.logger.debug("\nDatabase '{}' was already initialized".format(database_full_path))

            self._compress_and_encode_data_from_log_entry(log_entry)
            last_row_id = self._commit_and_update_checksum(database_conn, log_entry)
            self.logger.debug("\nRegistry {} inserted on database {}".format(last_row_id, database_full_path))

            event_dispatch_time = datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S.000")
            return EventDispatch(event_dispatch_time, last_row_id, log_entry, self.encode)

    def _commit_and_update_checksum(self, database_conn, log_entry):
        new_checksum = self._get_new_checksum(database_conn, log_entry.event_data)
        last_row_id = self._commit_new_entry(database_conn, log_entry)
        self._save_checksum(database_conn, new_checksum)
        return last_row_id

    def _get_new_checksum(self, database_conn, data):
        old_checksum = self._get_database_checksum(database_conn)
        return _get_unsigned_crc32(data, old_checksum)

    def _get_databases_list(self, period_to_search):
        files_list = glob.glob(os.path.join(self.audit_dir, DATABASE_NAME_TEMPLATE.format("[0-9]" * 8)))
        filtered_files_list = filter(lambda x: x[-12:-4] >= period_to_search, files_list)
        return sorted(filtered_files_list)

    def fetch_resync_events(self, resync_request):
        # type: (ResyncRequest) -> List[EventDispatch]
        log_message = "Receive Resync request: Date {} / RowId {}"
        self.logger.info(log_message.format(resync_request.period, resync_request.last_row_id))
        all_events = []
        for file_name in self._get_databases_list(resync_request.period):
            database_full_path = file_name

            with self._connect_database(database_full_path) as database_conn:
                event_types_str = ','.join("'{}'".format(x) for x in resync_request.event_types.split(','))
                formatted_query = FETCH_ENTRY_QUERY.format(event_types_str)

                params = {"period": resync_request.period, "row_id": resync_request.last_row_id}
                for row in database_conn.execute(formatted_query, params):
                    log_entry = self._create_log_entry_obj_with_query_result(row)
                    all_events.append(EventDispatch(row[7], row[0], log_entry, bool(row[3])))

        self.logger.info("Found {} event(s) to resync".format(len(all_events)))

        return all_events

    def _connect_database(self, database_full_path):
        try:
            return sqlite3.connect(database_full_path, self.conn_timeout)
        except Exception as _:
            self.logger.exception("Error opening Database {}".format(database_full_path))
            raise

    @staticmethod
    def _create_log_entry_obj_with_query_result(row):
        return LogEntry(
            business_period=row[1],
            event_type=row[2],
            created_at_gmt=row[6],
            event_data=row[4],
            order_id=row[5]
        )

    def _get_database_name_from_business_period(self, business_period):
        formatted_name = DATABASE_NAME_TEMPLATE.format(business_period)
        return self._get_database_full_path(formatted_name)

    def _get_database_full_path(self, file_name):
        return os.path.normpath(os.path.join(self.output_path, file_name).format(**os.environ))

    @staticmethod
    def _database_needs_initialization(conn):
        if not _table_exists(conn, "UUID"):
            return True

        if not _table_exists(conn, "AuditLog"):
            return True

        return False

    @staticmethod
    def _initialize_database(checksum, database_conn, uuid_data):
        database_conn.execute(CREATE_UUID_TABLE, (uuid_data,))

        database_conn.execute(CREATE_AUDITLOG_TABLE)
        LogRepository._save_checksum(database_conn, checksum)
        database_conn.commit()

    @staticmethod
    def _save_checksum(database_conn, checksum):
        database_conn.execute("PRAGMA user_version={};".format(checksum))
        database_conn.commit()

    @staticmethod
    def _get_database_checksum(database_conn):
        for row in database_conn.execute("PRAGMA user_version;"):
            return int(row[0])

    def _commit_new_entry(self, database_conn, log_entry):
        params = log_entry.__dict__.copy()
        params["encode"] = self.encode
        cursor = database_conn.cursor()
        cursor.execute(
            """
            INSERT INTO AuditLog(POSId,BusinessPeriod,EvtType,OrderId,CreatedAt,SessionId,Encoded,EvtData)
            VALUES(:pos_id,:business_period,:event_type,:order_id,:created_at_gmt,:session_id,:encode,:event_data);
            """, params
        )
        last_row_id = cursor.lastrowid
        database_conn.commit()
        return last_row_id

    def _compress_and_encode_data_from_log_entry(self, log_entry):
        compressed_data = self._compress_data(log_entry.event_data)
        if self.encode:
            log_entry.event_data = b64encode(compressed_data)
        else:
            log_entry.event_data = compressed_data

    @staticmethod
    def _compress_data(data):
        compress_obj = compressobj(Z_DEFAULT_COMPRESSION, DEFLATED, MAX_WBITS + 16)
        return compress_obj.compress(data) + compress_obj.flush()



import json

from application.interactor.apply_coupon import ApplyCouponInteractor
from application.interactor.apply_coupon.dto.request import (
    ApplyCouponRequest,
    ApplyCouponContext,
)
from messagebus import (
    DataType,
    Message,
    MessageBus,
    AckMessage,
    InternalErrorMessage,
)
from messageprocessorutil import UuidMessageProcessor
from .dto import ApplyCouponErrorCode
from ..dto import ResponseToken
from ...model.customexception import ErrorApplyingBenefit
from ...model.repository.benefit_applier.exception import (
    CouponAlreadyUsed,
    CouponLocked,
    CouponCustomError,
    BenefitApplierNotFound,
)


class ApplyCouponProcessor(UuidMessageProcessor):

    def __init__(
        self,        # type: ApplyCouponProcessor
        interactor,  # type: ApplyCouponInteractor
    ):
        # type: (...) -> None

        super(ApplyCouponProcessor, self).__init__()
        self.interactor = interactor

    def parse_data(
        self,        # type: ApplyCouponProcessor
        data,        # type: bytes
        event_type,  # type: str
    ):
        # type: (...) -> (int, int, str, int)

        parsed_data = json.loads(data)
        pos_id = int(parsed_data.get("posId"))
        originator_pos_id = int(parsed_data.get("originatorPosId") or pos_id)
        coupon_id = str(parsed_data["couponId"])
        user_auth_level = parsed_data.get("user_auth_level")
        if user_auth_level is None:
            user_auth_level = -1

        return ApplyCouponRequest(
            coupon_id=coupon_id,
            context=ApplyCouponContext(
                pos_id=pos_id,
                originator_pos_id=originator_pos_id,
                user_auth_level=user_auth_level
            )
        )

    def call_business(
            self,     # type: ApplyCouponProcessor
            request,  # type: ApplyCouponRequest
    ):
        # type: (...) -> None

        self.interactor.apply_coupon(apply_coupon_request=request)

    def format_response(
        self,         # type: ApplyCouponProcessor
        message_bus,  # type: MessageBus
        message,      # type: Message
        event,        # type: None
        request,      # type: ApplyCouponRequest
        response,     # type: None
    ):
        # type: (...) -> None

        message_bus.reply_message(
            message=message,
            reply_message=AckMessage()
        )

    def format_exception(
        self,         # type: ApplyCouponProcessor
        message_bus,  # type: MessageBus
        message,      # type: Message
        event,        # type: None
        input_obj,    # type: ApplyCouponRequest
        exception,    # type: Exception
    ):
        # type: (...) -> None

        exception_map = {
            CouponAlreadyUsed: (ApplyCouponErrorCode.COUPON_ALREADY_USED, None),
            CouponLocked: (ApplyCouponErrorCode.COUPON_LOCKED, None),
            CouponCustomError: (ApplyCouponErrorCode.CUSTOM_ERROR, self._build_data_from_coupon_custom_error),
            BenefitApplierNotFound: (ApplyCouponErrorCode.COUPON_NOT_FOUND, None),
            ErrorApplyingBenefit: (ApplyCouponErrorCode.NDISCOUNT_ERROR, self._build_data_from_ndiscount_error)
        }
        error_code, data_builder = exception_map.get(type(exception), (None, None))

        if error_code is not None:
            data = error_code.name
            if data_builder is not None:
                data += "\0" + data_builder(exception)

            message_bus.reply_message(
                message=message,
                reply_message=Message(
                    token=ResponseToken.TK_NCOUPON_ERROR,
                    data_type=DataType.param,
                    data=data,
                )
            )
        else:
            message_bus.reply_message(
                message=message,
                reply_message=InternalErrorMessage(exception),
            )

    def format_parse_exception(
        self,         # type: ApplyCouponProcessor
        message_bus,  # type: MessageBus
        message,      # type: Message
        event,        # type: None
        data,         # type: bytes
        exception,    # type: Exception
    ):
        # type: (...) -> None

        message_bus.reply_message(
            message=message,
            reply_message=InternalErrorMessage(exception),
        )

    @staticmethod
    def _build_data_from_coupon_custom_error(
        exception,  # type: CouponCustomError
    ):
        # type: (...) -> bytes

        return json.dumps({
            "message": {
                "pt": exception.message.pt,
                "en": exception.message.en,
                "es": exception.message.es,
            }
        }).encode("utf-8")

    @staticmethod
    def _build_data_from_ndiscount_error(
        exception,  # type: ErrorApplyingBenefit
    ):
        # type: (...) -> str

        return str(exception)



# -*- coding: utf-8 -*-
import datetime
import json
import sysactions

from plistlib import Dict
from _AposA8PrintLine import AposA8PrintLine
from _AposA8Align import AposA8Align
from _AposA8CommandType import AposA8CommandType
from _AposA8FiscalPrinter import AposA8FiscalPrinter
from common import PrinterCommon
from dateutil import tz
from msgbus import MBEasyContext
from _AposA8Helpers import APOS_DASH, APOS_NEWLINE, append_tax_data
from xml.etree import ElementTree as eTree

from old_helper import remove_xml_namespace
from clichehelper import ClicheHelper
from typing import Optional


class NfceAposA8FiscalPrinter(AposA8FiscalPrinter):
    def __init__(
        self,
        mb_context,                    # type: MBEasyContext
        print_customer_name,           # type: bool
        cliche_helper,                 # type: ClicheHelper
        qrcode_url,                    # type: str
        customer_text_field,           # type: str
        printer_common,                # type: PrinterCommon
        coupon_fiscal_parameters,      # type: Dict[str, bool]
        font_size_withdrawal_panel,    # type: str
    ):
        # type: (...) -> None

        super(NfceAposA8FiscalPrinter, self).__init__(
            mb_context,
            print_customer_name,
            cliche_helper,
            customer_text_field,
            coupon_fiscal_parameters,
            font_size_withdrawal_panel,
        )
        self.qrcode_check_url = qrcode_url
        self.satisfaction_survey_text = None
        self.customer_text_field = customer_text_field
        self.printer_common = printer_common

    def print_fiscal(
            self,
            pos_id,           # type: str
            order,            # type: eTree.Element
            tenders,          # type: list
            fiscal_request,   # type: eTree.Element
            fiscal_response,  # type: eTree.Element
            total_taxes,      # type: dict
            customer_name,    # type: str
            url_sefaz="",     # type: str
    ):
        # type: (...) -> str

        model = sysactions.get_model(pos_id)
        fiscal_xml = remove_xml_namespace(eTree.tostring(fiscal_request))

        tp_emiss = fiscal_xml.find(".//tpEmis").text

        if tp_emiss == "9":
            report = self._fiscal_report(
                order=order,
                tenders=tenders,
                fiscal_request=fiscal_request,
                fiscal_response=fiscal_response,
                total_taxes=total_taxes,
                via="Via Estabelecimento",
                customer_name=customer_name,
                url_sefaz=self.qrcode_check_url,
                model=model,
            )
            report.append(APOS_NEWLINE)
            report.extend(
                self._fiscal_report(
                    order=order,
                    tenders=tenders,
                    fiscal_request=fiscal_request,
                    fiscal_response=fiscal_response,
                    total_taxes=total_taxes,
                    via="Via Consumidor",
                    customer_name=customer_name,
                    url_sefaz=self.qrcode_check_url,
                    model=model,
                )
            )
        else:
            report = self._fiscal_report(
                order=order,
                tenders=tenders,
                fiscal_request=fiscal_request,
                fiscal_response=fiscal_response,
                total_taxes=total_taxes,
                via="",
                customer_name=customer_name,
                url_sefaz=self.qrcode_check_url,
                model=model,
            )

        return json.dumps(report, default=lambda x: x.__dict__)

    def print_cancel(self, pos_id, data):
        # type: (str, str) -> None
        pass

    def _fiscal_report(
            self,
            order,            # type: eTree.Element
            tenders,          # type: list
            fiscal_request,   # type: eTree.Element
            fiscal_response,  # type: eTree.Element
            total_taxes,      # type: dict
            via,              # type: str
            customer_name,    # type: str
            url_sefaz,        # type: str
            model=None,       # type: Optional[eTree.Element]
    ):
        # type: (...) -> list

        fiscal_xml = remove_xml_namespace(eTree.tostring(fiscal_request))

        report = []

        self._append_cliche(report)

        self._append_header(report)

        is_contingency = fiscal_xml.find(".//tpEmis").text == "9"

        if is_contingency:
            self._append_contingency_warning(report)

        self._append_coupon_body(
            customer_name=customer_name,
            model=model,
            order=order,
            report=report,
            tenders=tenders,
            printer_common=self.printer_common,
        )

        self._append_sefaz_url_and_key(report, fiscal_xml, url_sefaz)

        self._append_cpf_cnpj(report, fiscal_xml)

        self._append_nfce_data(report, fiscal_xml)

        if is_contingency:
            report.append(AposA8PrintLine(AposA8Align.CENTER, via, AposA8CommandType.PRINT_TEXT))

        response_xml = None
        if fiscal_response is not None:
            response_xml = remove_xml_namespace(eTree.tostring(fiscal_response))

        self._append_auth_protocol(report, response_xml)

        self._append_homolog_warning(report, fiscal_xml)

        if is_contingency:
            report.append(AposA8PrintLine(AposA8Align.CENTER, "EMITIDA EM CONTINGENCIA", AposA8CommandType.PRINT_TEXT))
            report.append(AposA8PrintLine(AposA8Align.CENTER, "Pendente de autorizacao", AposA8CommandType.PRINT_TEXT))

        self._append_qr_code(report, fiscal_xml)

        self._append_response_protocol_information(report, response_xml)

        append_tax_data(report, total_taxes)

        self._append_customer_text_field(report)

        return report

    @staticmethod
    def _append_response_protocol_information(report, response_xml):
        if response_xml is not None:
            message_code_tag = response_xml.find(".//infProt/cMsg")
            message_code = message_code_tag.text if message_code_tag is not None else None
            message_text_tag = response_xml.find(".//infProt/xMsg")
            message_text = message_text_tag.text if message_text_tag is not None else None
            if message_code == '200' and message_text not in [None, '', ' ']:
                text_to_print = message_text.replace('|', '\n').strip()
                report.append(AposA8PrintLine(AposA8Align.CENTER, text_to_print, AposA8CommandType.PRINT_TEXT))

    def _append_auth_protocol(self, report, response_xml):
        if response_xml is not None:
            protocolo_element = response_xml.find(".//nProt").text
            if protocolo_element is not None:
                auth_protocol_text = "Protocolo de autorizacao: " + protocolo_element
                report.append(AposA8PrintLine(AposA8Align.CENTER, auth_protocol_text, AposA8CommandType.PRINT_TEXT))

                protocol_date = response_xml.find(".//dhRecbto").text
                protocol_date_text = "Data de autorizacao: " + self._formata_data(protocol_date)
                report.append(AposA8PrintLine(AposA8Align.CENTER, protocol_date_text, AposA8CommandType.PRINT_TEXT))

    @staticmethod
    def _append_homolog_warning(report, fiscal_xml):
        tp_amb = fiscal_xml.find(".//tpAmb").text
        if tp_amb == "2":
            text_to_print = "EMITIDA EM HOMOLOGACAO - SEM VALOR FISCAL"
            report.append(AposA8PrintLine(AposA8Align.CENTER, text_to_print, AposA8CommandType.PRINT_TEXT))

    @staticmethod
    def _append_qr_code(report, fiscal_xml):
        url = fiscal_xml.find(".//qrCode").text.strip()
        report.append(AposA8PrintLine(AposA8Align.CENTER, url, AposA8CommandType.PRINT_QR_CODE))

    def _append_nfce_data(self, report, fiscal_xml):
        numero = fiscal_xml.find(".//nNF").text
        serie = fiscal_xml.find(".//serie").text
        data_emissao_xml = fiscal_xml.find(".//dhEmi").text
        data_emissao_str = self._formata_data(data_emissao_xml)
        nfce_data = "NFC-e Numero %09d  Serie %03d  %s" % (int(numero), int(serie), data_emissao_str)
        report.append(AposA8PrintLine(AposA8Align.CENTER, nfce_data, AposA8CommandType.PRINT_TEXT))

    @staticmethod
    def _append_cpf_cnpj(report, fiscal_xml):
        cpf_cnpj = ""
        cpf = fiscal_xml.find(".//dest/CPF")
        if cpf is not None:
            cpf_cnpj = cpf.text
        else:
            cnpj = fiscal_xml.find(".//dest/CNPJ")
            if cnpj is not None:
                cpf_cnpj = cnpj.text
        cpf_cnpj_text = "CONSUMIDOR %s" % ("NAO INFORMADO" if not cpf_cnpj else ("- CPF " + cpf_cnpj))
        report.append(AposA8PrintLine(AposA8Align.CENTER, cpf_cnpj_text, AposA8CommandType.PRINT_TEXT))

    @staticmethod
    def _append_sefaz_url_and_key(report, fiscal_xml, url_sefaz):
        text_to_print = "Consulte pela Chave de Acesso em"
        report.append(AposA8PrintLine(AposA8Align.CENTER, text_to_print, AposA8CommandType.PRINT_TEXT))
        report.append(AposA8PrintLine(AposA8Align.CENTER, url_sefaz, AposA8CommandType.PRINT_TEXT))
        key = fiscal_xml.find(".//infNFe").attrib['Id'][3:]
        key_text = ' '.join(key[i:i + 4] for i in range(0, 44, 4))
        split_key = key_text[:24] + '\n' + key_text[25:]
        report.append(AposA8PrintLine(AposA8Align.CENTER, split_key, AposA8CommandType.PRINT_TEXT))

    @staticmethod
    def _append_contingency_warning(report):
        report.append(APOS_NEWLINE)
        report.append(AposA8PrintLine(AposA8Align.CENTER, "EMITIDA EM CONTINGENCIA", AposA8CommandType.PRINT_TEXT))
        report.append(AposA8PrintLine(AposA8Align.CENTER, "Pendente de autorizacao", AposA8CommandType.PRINT_TEXT))
        report.append(APOS_DASH)

    @staticmethod
    def _append_header(report):
        header_message_1 = "Documento Auxiliar da Nota Fiscal"
        header_message_2 = "de Consumidor Eletronica"
        report.append(AposA8PrintLine(AposA8Align.CENTER, header_message_1, AposA8CommandType.PRINT_TEXT))
        report.append(AposA8PrintLine(AposA8Align.CENTER, header_message_2, AposA8CommandType.PRINT_TEXT))
        report.append(APOS_DASH)

    @staticmethod
    def _formata_data(data, mascara="%d/%m/%Y %H:%M:%S"):
        data_emissao_date_utc = data[:19]
        data_emissao_date_utc = datetime.datetime.strptime(data_emissao_date_utc, "%Y-%m-%dT%H:%M:%S")
        data_emissao_offset = int(data[20:22]) * 60 + int(data[23:])
        if data[19] == "+":
            data_emissao_offset *= -1
        data_emissao_date_utc = data_emissao_date_utc + datetime.timedelta(minutes=data_emissao_offset)
        from_zone = tz.tzutc()
        to_zone = tz.tzlocal()

        data_emissao_date = data_emissao_date_utc.replace(tzinfo=from_zone)
        # Convert time zone
        data_emissao_date = data_emissao_date.astimezone(to_zone)
        data_emissao_str = data_emissao_date.strftime(mascara)
        return data_emissao_str



import json

from typing import Tuple

from application.interactor import BurnVoucherInteractor, UnlockVoucherInteractor
from application.model import AsyncUuidMessageProcessor
from application.model.configuration import Configurations
from messagebus import DefaultToken, DataType, Message


class UnlockVoucherProcessor(AsyncUuidMessageProcessor):
    def __init__(self, configs, interactor):
        # type: (Configurations, UnlockVoucherInteractor) -> None

        super(UnlockVoucherProcessor, self).__init__()
        self.logger = configs.logger
        self.configs = configs
        self.interactor = interactor

    def parse_data(self, data, event_type):
        # type: (str, str) -> str

        request_json = json.loads(data.decode("utf-8"))
        return str(request_json["benefitId"]), int(request_json["posId"])

    def call_business(
        self,
        data_tuple,  # type: Tuple[str, int]
    ):
        # type: (...) -> None

        benefit_id = data_tuple[0]
        pos_id = data_tuple[1]

        self.interactor.unlock_benefit(
            benefit_id=benefit_id,
            pos_id=pos_id,
        )

    def format_response(self, message_bus, message, event, request, response):
        message_bus.reply_message(
            message=message,
            reply_message=Message(
                token=DefaultToken.TK_SYS_ACK,
                data_type=DataType.string,
            ),
        )

    def format_exception(self, message_bus, message, event, input_obj, exception):
        message_bus.reply_message(message, Message(DefaultToken.TK_SYS_NAK, DataType.param, str(exception)))

    def format_parse_exception(self, message_bus, message, event, data, exception):
        message_bus.reply_message(message, Message(DefaultToken.TK_SYS_NAK, DataType.param, str(exception)))



class Action(object):
    def __init__(self, obj):
        self.order = int(obj.get("order")) if obj.get("order") else None
        self.name = obj.get("name")
        self.value = obj.get("value")
        self.times_to_apply = None
        self.seat = obj.get("seat", None)
        self.max_value = obj.get(
            "maxValue",
            None,
        )
        self.tender_id = int(obj["tenderId"]) if "tenderId" in obj and obj["tenderId"] is not None else None



#!/usr/bin/env python3
#
# Working with threading and pySerial
#
# This file is part of pySerial. https://github.com/pyserial/pyserial
# (C) 2015-2016 Chris Liechti <cliechti@gmx.net>
#
# SPDX-License-Identifier:    BSD-3-Clause
"""\
Support threading with serial ports.
"""
from __future__ import absolute_import

import serial
import threading


class Protocol(object):
    """\
    Protocol as used by the ReaderThread. This base class provides empty
    implementations of all methods.
    """

    def connection_made(self, transport):
        """Called when reader thread is started"""

    def data_received(self, data):
        """Called with snippets received from the serial port"""

    def connection_lost(self, exc):
        """\
        Called when the serial port is closed or the reader loop terminated
        otherwise.
        """
        if isinstance(exc, Exception):
            raise exc


class Packetizer(Protocol):
    """
    Read binary packets from serial port. Packets are expected to be terminated
    with a TERMINATOR byte (null byte by default).

    The class also keeps track of the transport.
    """

    TERMINATOR = b'\0'

    def __init__(self):
        self.buffer = bytearray()
        self.transport = None

    def connection_made(self, transport):
        """Store transport"""
        self.transport = transport

    def connection_lost(self, exc):
        """Forget transport"""
        self.transport = None
        super(Packetizer, self).connection_lost(exc)

    def data_received(self, data):
        """Buffer received data, find TERMINATOR, call handle_packet"""
        self.buffer.extend(data)
        while self.TERMINATOR in self.buffer:
            packet, self.buffer = self.buffer.split(self.TERMINATOR, 1)
            self.handle_packet(packet)

    def handle_packet(self, packet):
        """Process packets - to be overridden by subclassing"""
        raise NotImplementedError('please implement functionality in handle_packet')


class FramedPacket(Protocol):
    """
    Read binary packets. Packets are expected to have a start and stop marker.

    The class also keeps track of the transport.
    """

    START = b'('
    STOP = b')'

    def __init__(self):
        self.packet = bytearray()
        self.in_packet = False
        self.transport = None

    def connection_made(self, transport):
        """Store transport"""
        self.transport = transport

    def connection_lost(self, exc):
        """Forget transport"""
        self.transport = None
        self.in_packet = False
        del self.packet[:]
        super(FramedPacket, self).connection_lost(exc)

    def data_received(self, data):
        """Find data enclosed in START/STOP, call handle_packet"""
        for byte in serial.iterbytes(data):
            if byte == self.START:
                self.in_packet = True
            elif byte == self.STOP:
                self.in_packet = False
                self.handle_packet(bytes(self.packet)) # make read-only copy
                del self.packet[:]
            elif self.in_packet:
                self.packet.extend(byte)
            else:
                self.handle_out_of_packet_data(byte)

    def handle_packet(self, packet):
        """Process packets - to be overridden by subclassing"""
        raise NotImplementedError('please implement functionality in handle_packet')

    def handle_out_of_packet_data(self, data):
        """Process data that is received outside of packets"""
        pass


class LineReader(Packetizer):
    """
    Read and write (Unicode) lines from/to serial port.
    The encoding is applied.
    """

    TERMINATOR = b'\r\n'
    ENCODING = 'utf-8'
    UNICODE_HANDLING = 'replace'

    def handle_packet(self, packet):
        self.handle_line(packet.decode(self.ENCODING, self.UNICODE_HANDLING))

    def handle_line(self, line):
        """Process one line - to be overridden by subclassing"""
        raise NotImplementedError('please implement functionality in handle_line')

    def write_line(self, text):
        """
        Write text to the transport. ``text`` is a Unicode string and the encoding
        is applied before sending ans also the newline is append.
        """
        # + is not the best choice but bytes does not support % or .format in py3 and we want a single write call
        self.transport.write(text.encode(self.ENCODING, self.UNICODE_HANDLING) + self.TERMINATOR)


class ReaderThread(threading.Thread):
    """\
    Implement a serial port read loop and dispatch to a Protocol instance (like
    the asyncio.Protocol) but do it with threads.

    Calls to close() will close the serial port but it is also possible to just
    stop() this thread and continue the serial port instance otherwise.
    """

    def __init__(self, serial_instance, protocol_factory):
        """\
        Initialize thread.

        Note that the serial_instance' timeout is set to one second!
        Other settings are not changed.
        """
        super(ReaderThread, self).__init__()
        self.daemon = True
        self.serial = serial_instance
        self.protocol_factory = protocol_factory
        self.alive = True
        self._lock = threading.Lock()
        self._connection_made = threading.Event()
        self.protocol = None

    def stop(self):
        """Stop the reader thread"""
        self.alive = False
        if hasattr(self.serial, 'cancel_read'):
            self.serial.cancel_read()
        self.join(2)

    def run(self):
        """Reader loop"""
        if not hasattr(self.serial, 'cancel_read'):
            self.serial.timeout = 1
        self.protocol = self.protocol_factory()
        try:
            self.protocol.connection_made(self)
        except Exception as e:
            self.alive = False
            self.protocol.connection_lost(e)
            self._connection_made.set()
            return
        error = None
        self._connection_made.set()
        while self.alive and self.serial.is_open:
            try:
                # read all that is there or wait for one byte (blocking)
                data = self.serial.read(self.serial.in_waiting or 1)
            except serial.SerialException as e:
                # probably some I/O problem such as disconnected USB serial
                # adapters -> exit
                error = e
                break
            else:
                if data:
                    # make a separated try-except for called user code
                    try:
                        self.protocol.data_received(data)
                    except Exception as e:
                        error = e
                        break
        self.alive = False
        self.protocol.connection_lost(error)
        self.protocol = None

    def write(self, data):
        """Thread safe writing (uses lock)"""
        with self._lock:
            return self.serial.write(data)

    def close(self):
        """Close the serial port and exit reader thread (uses lock)"""
        # use the lock to let other threads finish writing
        with self._lock:
            # first stop reading, so that closing can be done on idle port
            self.stop()
            self.serial.close()

    def connect(self):
        """
        Wait until connection is set up and return the transport and protocol
        instances.
        """
        if self.alive:
            self._connection_made.wait()
            if not self.alive:
                raise RuntimeError('connection_lost already called')
            return (self, self.protocol)
        else:
            raise RuntimeError('already stopped')

    # - -  context manager, returns protocol

    def __enter__(self):
        """\
        Enter context handler. May raise RuntimeError in case the connection
        could not be created.
        """
        self.start()
        self._connection_made.wait()
        if not self.alive:
            raise RuntimeError('connection_lost already called')
        return self.protocol

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Leave context: close port"""
        self.close()


# - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
# test
if __name__ == '__main__':
    # pylint: disable=wrong-import-position
    import sys
    import time
    import traceback

    #~ PORT = 'spy:///dev/ttyUSB0'
    PORT = 'loop://'

    class PrintLines(LineReader):
        def connection_made(self, transport):
            super(PrintLines, self).connection_made(transport)
            sys.stdout.write('port opened\n')
            self.write_line('hello world')

        def handle_line(self, data):
            sys.stdout.write('line received: {!r}\n'.format(data))

        def connection_lost(self, exc):
            if exc:
                traceback.print_exc(exc)
            sys.stdout.write('port closed\n')

    ser = serial.serial_for_url(PORT, baudrate=115200, timeout=1)
    with ReaderThread(ser, PrintLines) as protocol:
        protocol.write_line('hello')
        time.sleep(2)

    # alternative usage
    ser = serial.serial_for_url(PORT, baudrate=115200, timeout=1)
    t = ReaderThread(ser, PrintLines)
    t.start()
    transport, protocol = t.connect()
    protocol.write_line('hello')
    time.sleep(2)
    t.close()



# encoding: utf-8

from typing import Dict


class JustificationDiscountBodyDto(object):
    def __init__(
        self,
        justifications,     # type: Dict
    ):
        # type: (...) -> None

        self.justifications = justifications
